@@ -2,6 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -33,18 +34,14 @@
 #ifndef _defs_h
 #define _defs_h
 
-#ifndef _colodefs_h
-#include "colodefs.h"
-#endif
-
 //--------------------
 // Some technical definitions...
 //
 #ifndef TRUE
-#define TRUE (1==1)
+#define TRUE (1)
 #endif
 #ifndef FALSE
-#define FALSE (1==0)
+#define FALSE (0)
 #endif
 
 #define ERR 	-1
@@ -86,7 +83,7 @@ enum
 
 enum
   {
-    COLLISION_TYPE_NONE = -1 , 
+    COLLISION_TYPE_NONE = 1 , 
     COLLISION_TYPE_RECTANGLE = 2 ,
     COLLISION_TYPE_CIRCLE = 3 
   };
@@ -101,14 +98,9 @@ enum
     PART_GROUP_WEAPONARM = 5 ,
     ALL_PART_GROUPS = 6
   };
-enum
-  {
-    TUX_IMAGE_UPDATE_EVERYTHING_AT_ONCE = -133 ,
-    TUX_IMAGE_UPDATE_CONTINUOUSLY = -134 
-  };
 #define NOT_LOADED_MARKER "nothing_loaded"
 
-#define ENEMY_ROTATION_MODELS_AVAILABLE 34
+#define ENEMY_ROTATION_MODELS_AVAILABLE 35
 #define ROTATION_ANGLES_PER_ROTATION_MODEL 8
 
 #define MAX_ENEMY_MOVEMENT_PHASES 50
@@ -124,11 +116,13 @@ enum
 #define NUMBER_OF_SHADOW_IMAGES 20
 
 
-#define MAX_OBSTACLES_ON_MAP 3000
+#define MAX_OBSTACLES_ON_MAP 4000
 #define MAX_OBSTACLE_NAMES_PER_LEVEL 100
+#define MAX_OBSTACLE_STATES_PER_LEVEL 300
 #define MAX_OBSTACLE_DESCRIPTIONS_PER_LEVEL 100
 
-#define FLOOR_TILES_VISIBLE_AROUND_TUX 11
+#define FLOOR_TILES_VISIBLE_AROUND_TUX ((GameConfig . screen_width >= 1024 ? 12 : GameConfig . screen_width >= 800 ? 9 : 7))
+#define MAX_FLOOR_TILES_VISIBLE_AROUND_TUX 12
 // #define MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE 4
 #define MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE 5
 #define MAX_ITEMS_PER_LEVEL 300
@@ -142,48 +136,42 @@ enum
 #define NUMBER_OF_ITEM_PICTURES 70
 #define NUMBER_OF_MOUSE_CURSOR_PICTURES 2
 
-#define ITEM_DROP_PERCENTAGE 25
-#define GOLD_DROP_PERCENTAGE 15
+#define ITEM_DROP_PERCENTAGE 15
+#define GOLD_DROP_PERCENTAGE 20
+#define ARMOUR_DURABILITYLOSS_PERCENTAGE_WHEN_HIT 20
+#define WEAPON_DURABILITYLOSS_PERCENTAGE_WHEN_USED 15
 
 #define MAX_DIALOGUE_OPTIONS_IN_ROSTER 100
 #define MAX_REPLIES_PER_OPTION 100
 #define MAX_SUBTITLES_N_SAMPLES_PER_DIALOGUE_OPTION 20
-#define MAX_EXTRAS_PER_OPTION 10
 #define CHAT_DEBUG_LEVEL 1
 
 #define MAX_BIG_SCREEN_MESSAGES 10
 
-
-#ifndef _itemdefs_h
-#include "itemdefs.h"
-#endif
-
-#define WAR_BOT 1
-#define SNIPER_BOT 2
-#define MIND_BOT 3
-
 #define NUMBER_OF_SKILL_LEVELS 10
 #define NUMBER_OF_SKILL_PAGES 8
 #define NUMBER_OF_SKILLS_PER_SKILL_PAGE 5
-#define NUMBER_OF_SKILLS ( NUMBER_OF_SKILLS_PER_SKILL_PAGE * NUMBER_OF_SKILL_PAGES )
-
 #define MAX_ACTIVE_SPELLS 100
+#define MAX_NUMBER_OF_PROGRAMS 50
 
+enum skill_forms
+{
+    PROGRAM_FORM_IMMEDIATE = 10, //instant hit whatever behind cursor
+    PROGRAM_FORM_RADIAL, //nova
+    PROGRAM_FORM_BULLET, //fire a bullet
+    PROGRAM_FORM_SELF, //instant hit player
+};
 
 enum _global_ingame_modes
 {
     GLOBAL_INGAME_MODE_IDENTIFY = 7123 ,
     GLOBAL_INGAME_MODE_NORMAL = 7124 ,
     GLOBAL_INGAME_MODE_EXAMINE = 7125 ,
-    GLOBAL_INGAME_MODE_LOOT = 7126 ,
     GLOBAL_INGAME_MODE_SCROLL_UP = 7127, 
     GLOBAL_INGAME_MODE_SCROLL_DOWN = 7128 ,
     GLOBAL_INGAME_MODE_REPAIR = 7129 ,
-    GLOBAL_INGAME_MODE_UNLOCK = 7130 ,
-    GLOBAL_INGAME_MODE_TALK = 7131 ,
-    GLOBAL_INGAME_MODE_FIRST_AID = 7132 ,
-    GLOBAL_INGAME_MODE_ATTACK = 7133 ,
-    GLOBAL_INGAME_MODE_PICKPOCKET = 7134 ,
+    GLOBAL_INGAME_MODE_SELECT_TOOL = 7130 ,
+    GLOBAL_INGAME_MODE_DRAGDROP_TOOL = 7131 ,
 };
 
 enum _game_config_screen_visible_parameters
@@ -194,37 +182,6 @@ enum _game_config_screen_visible_parameters
     GAME_CONFIG_SCREEN_VISIBLE_SKILL_EXPLANATION = 8004 ,
 };
 
-enum _spell_names
-{
-  SPELL_TRANSFERMODE = 0, // 0
-  SPELL_LOOT_CHEST_OR_DEAD_BODY, 
-  SPELL_REPAIR_SKILL, // 
-  SPELL_WEAPON, // 
-  SPELL_IDENTIFY_SKILL, // 
-  SPELL_FORCE_TO_ENERGY, // 
-  SPELL_TELEPORT_HOME, // 
-  SPELL_FIREY_BOLT, // 
-  SPELL_COLD_BOLT, // 
-  SPELL_FORCE_EXPLOSION_CIRCLE, // 
-  SPELL_POISON_BOLT, // 
-  SPELL_PARALYZE_BOLT, // 
-  SPELL_DETECT_ITEM, // 
-  SPELL_RADIAL_EMP_WAVE, //
-  SPELL_RADIAL_VMX_WAVE, // 
-  SPELL_RADIAL_FIRE_WAVE, // 
-  SPELL_EXTRACT_PLASMA_TRANSISTORS,
-  SPELL_EXTRACT_SUPERCONDUCTORS,
-  SPELL_EXTRACT_ANTIMATTER_CONVERTERS,
-  SPELL_EXTRACT_ENTROPY_INVERTERS,
-  SPELL_EXTRACT_TACHYON_CONDENSATORS,
-  SPELL_FORCE_EXPLOSION_RAY, // 
-};
-
-//--------------------
-// This is old and should go out, but that must be done from at home, 
-// cause here I can't check if the code is still running without it
-//
-#define FILENAME_LEN	5000
 
 #define END_OF_OFFSET_FILE_STRING "** End of iso_image offset file **"
 #define START_OF_OFFSET_FILE_STRING "** Start of iso_image offset file **"
@@ -248,37 +205,39 @@ enum _spell_names
 
 // The flags for AssembleCombatWindow are:
 enum
-{ ONLY_SHOW_MAP = 1 , DO_SCREEN_UPDATE = 2 , ONLY_SHOW_MAP_AND_TEXT = 4 , SHOW_GRID = 8 , SHOW_TUX_AND_ENEMIES = 16 , SHOW_TEXT = 32 , SHOW_ITEMS = 64 , OMIT_OBSTACLES = 128 , OMIT_TUX = 256 , OMIT_ENEMIES = 512 , SHOW_OBSTACLE_NAMES = 1024 , ZOOM_OUT = 2048 , OMIT_BLASTS = 4096 , USE_OWN_MOUSE_CURSOR = 8192 , SKIP_LIGHT_RADIUS = (2*8192) };
+{ ONLY_SHOW_MAP = 1 , DO_SCREEN_UPDATE = 2 , ONLY_SHOW_MAP_AND_TEXT = 4 , SHOW_GRID = 8 , SHOW_TUX_AND_ENEMIES = 16 , SHOW_TEXT = 32 , SHOW_ITEMS = 64 , OMIT_OBSTACLES = 128 , OMIT_TUX = 256 , OMIT_ENEMIES = 512 , SHOW_OBSTACLE_NAMES = 1024 , ZOOM_OUT = 2048 , OMIT_BLASTS = 4096 , USE_OWN_MOUSE_CURSOR = 8192 , SKIP_LIGHT_RADIUS = 16384, NO_CURSOR = 32768 };
+
+// Flags for obstacles
+enum 
+{
+	/* Obstacle groups */
+	IS_WALL = 1,
+
+	/* Walls */
+	IS_HORIZONTAL = 2,
+	IS_VERTICAL = 4,
+
+	/* Obstacle specs */
+	BLOCKS_VISION_TOO = 8,			/* Light will not pass through this obstacle */
+	IS_SMASHABLE = 16,
+	DROPS_RANDOM_TREASURE = 32,
+	NEEDS_PRE_PUT = 64,
+	GROUND_LEVEL = 128,			/* This obstacle has a small height */
+	IS_WALKABLE = 256			/* A bot can traverse this obstacle */
+};
+
+
+
 
 //--------------------
 // This controls the zoom factor for the automap.  Since this uses
 // a different update policy than the level editor, even strong zoom
 // will not be a problem here...
 //
-#define LEVEL_EDITOR_ZOOM_OUT_FACT 3.0
+#define LEVEL_EDITOR_ZOOM_OUT_FACT 3
+#define ONE_OVER_LEVEL_EDITOR_ZOOM_OUT_FACT 0.3333333333333333333
 
-//--------------------
-// This controls the zoom factor for the automap.  Since this uses
-// a different update policy than the level editor, even strong zoom
-// will not be a problem here...
-//
-// However some other considerations must be taken into account:  OPENGL
-// DOES IN GENERAL NOT WORK WELL ANY MORE WITH TEXTURES LARGER THAN 1024
-// AND SOMETIMES EVEN NOT WELL WITH ANYTHING > 256 (old vodoo cards!!!)
-// EVEN GEFORCE II HAS PROBLEMS WITH 2048 TEXTURE SIZE.
-//
-// Therefore some sanity-factor is introduced here:  it will scale down
-// the internal automap texture but scale it up again when the acutal
-// part-transparent blit is being done.  Using 0.25 for the sanity factor
-// should be the safest bet.  However quality is poor:  you get lots of
-// rectangular blocks where a wall/door/tree would be. 
-// So we're using larger textures, assuming that most people will have 
-// decen graphics adapters. Vodoo users can recompile the game for their needs...
-// 
 #define AUTOMAP_ZOOM_OUT_FACT 8.0
-#define AUTOMAP_SANITY_FACTOR 1
-#define AUTOMAP_TEXTURE_WIDTH (2048*AUTOMAP_SANITY_FACTOR)
-#define AUTOMAP_TEXTURE_HEIGHT (1024*AUTOMAP_SANITY_FACTOR)
 
 #define LIGHT_RADIUS_STRETCH_TEXTURE_WIDTH 64
 #define LIGHT_RADIUS_STRETCH_TEXTURE_HEIGHT 64
@@ -331,7 +290,6 @@ enum
 // #define LEVEL_EDITOR_BANNER_CODE6 19 
 // #define LEVEL_EDITOR_BANNER_CODE7 20 
 #define FREEDROID_LOADING_PICTURE_CODE 21
-#define MOUSE_BUTTON_CHA_BACKGROUND_PICTURE_CODE 22
 #define MOUSE_BUTTON_INV_BACKGROUND_PICTURE_CODE 23
 #define MOUSE_BUTTON_SKI_BACKGROUND_PICTURE_CODE 24
 #define MOUSE_BUTTON_PLUS_BACKGROUND_PICTURE_CODE 25
@@ -365,8 +323,8 @@ enum
 #define BLOCKMEM  		INITIAL_BLOCK_WIDTH*INITIAL_BLOCK_HEIGHT
 #define MAX_AMOUNT_OF_TEXTURES_WE_WILL_USE 10000
 
-#define SCROLLSTARTX		User_Rect.x
-#define SCROLLSTARTY		GameConfig . screen_height
+#define TABCHAR 'W'
+#define TABWIDTH 8
 
 #define INITIAL_DIGIT_LENGTH (8*2)
 #define INITIAL_DIGIT_HEIGHT (9*2)
@@ -382,7 +340,7 @@ enum
 #define LEFT_INFO_Y	(10*2)
 #define RIGHT_INFO_X	(242*2)
 #define RIGHT_INFO_Y	(8*2)
-#define LEFT_TEXT_LEN 10
+#define LEFT_TEXT_LEN 20
 #define RIGHT_TEXT_LEN 6
 #define TEXT_STRETCH (6.0/5.0)
 
@@ -421,13 +379,15 @@ enum _sounds
 // occur periodically.  These constants specify which intervals are to be used
 // for these periodic happenings...
 #define CRY_SOUND_INTERVAL 2
-#define TRANSFER_SOUND_INTERVAL 1.1
-
-// #define MAX_PLAYERS             5
-#define MAX_PLAYERS             1
-#define MAX_PLAYERS_AT_MOST     5
-#define PLAYER_NR_0             0
 
+/* Those are the codes used internally to 
+ * store the state of the dialogs with 
+ * those various characters. When adding 
+ * a character, you are going to break existing
+ * savegames unless it is added at the end.
+ * Replace a PERSON_PLACEHOLDER with your new 
+ * name to minimize damage on existing savegames.
+ */
 enum _non_player_characters
   {
     PERSON_CHA = 0,
@@ -438,9 +398,11 @@ enum _non_player_characters
     PERSON_DUNCAN,
     PERSON_PENDRAGON,
     PERSON_614,
+    PERSON_614_CRYO,
     PERSON_MER,
+    PERSON_TUTORIALTOM,
     PERSON_FRANCIS,
-    PERSON_ERNIE,
+    PERSON_FIRMWARE_SERVER,
     PERSON_BENJAMIN,
     PERSON_BENDER,
     PERSON_SPENCER,
@@ -451,11 +413,9 @@ enum _non_player_characters
     PERSON_MICHELANGELO,
     PERSON_SKIPPY,
     PERSON_STANDARD_OLD_TOWN_GATE_GUARD,
-    PERSON_STANDARD_NEW_TOWN_GATE_GUARD,
     PERSON_OLD_TOWN_GATE_GUARD_LEADER,
     PERSON_STANDARD_MS_FACILITY_GATE_GUARD,
     PERSON_MS_FACILITY_GATE_GUARD_LEADER,
-    PERSON_HEA,
     PERSON_STANDARD_BOT_AFTER_TAKEOVER,
     PERSON_BRUCE,
     PERSON_SUBDIALOG_DUMMY,
@@ -465,6 +425,27 @@ enum _non_player_characters
     PERSON_KEVIN,
     PERSON_LUKAS,
     PERSON_JASMINE,
+    PERSON_SADD,
+    PERSON_SACD,
+    PERSON_TANIA,
+    PERSON_KOAN,
+    PERSON_BORIS,
+    PERSON_LINA,
+    PERSON_SERGE,
+    PERSON_TEST_DROID,
+    PERSON_MSCD,
+
+    PERSON_PLACEHOLDER1,
+    PERSON_PLACEHOLDER2,
+    PERSON_PLACEHOLDER3,
+    PERSON_PLACEHOLDER4,
+    PERSON_PLACEHOLDER5,
+    PERSON_PLACEHOLDER6,
+    PERSON_PLACEHOLDER7,
+    PERSON_PLACEHOLDER8,
+    PERSON_PLACEHOLDER9,
+    PERSON_PLACEHOLDER10,
+
     LAST_PERSON
   };
 #define MAX_PERSONS             (LAST_PERSON)
@@ -485,39 +466,6 @@ enum
     SKI_SCREEN_TOGGLE_BUTTON_YELLOW, 
     UP_BUTTON,
     DOWN_BUTTON,
-    LEFT_BUTTON,
-    RIGHT_BUTTON,
-    MAP_EXIT_BUTTON,
-    MAP_UNLOCK_BUTTON_GREEN,
-    MAP_UNLOCK_BUTTON_YELLOW,
-    MAP_UNLOCK_BUTTON_RED,
-    MAP_GUNOFF_BUTTON_GREEN,
-    MAP_GUNOFF_BUTTON_YELLOW,
-    MAP_GUNOFF_BUTTON_RED,
-    MAP_SECURITYMIDDLE_BUTTON,
-    MAP_SECURITYLEFT_BUTTON,
-    MAP_SECURITYRIGHT_BUTTON,
-    MAP_PASSWORDMIDDLE_BUTTON,
-    MAP_PASSWORDLEFT_BUTTON,
-    MAP_PASSWORDRIGHT_BUTTON,
-    MAP_REQUEST_ENERGY_RATION_GREEN_BUTTON,
-    MAP_REQUEST_ENERGY_RATION_RED_BUTTON,
-    MAP_READ_EMAIL_GREEN_BUTTON,
-    MAP_READ_EMAIL_RED_BUTTON,
-    MAP_GUN_TYPE_1_BUTTON_RED,
-    MAP_GUN_TYPE_2_BUTTON_RED,
-    MAP_GUN_TYPE_3_BUTTON_RED,
-    MAP_GUN_TYPE_4_BUTTON_RED,
-    MAP_GUNON_BUTTON_GREEN,
-    MAP_GUNON_BUTTON_YELLOW,
-    MAP_GUNON_BUTTON_RED,
-    MAP_GUN_TYPE_1_BUTTON_YELLOW,
-    MAP_GUN_TYPE_2_BUTTON_YELLOW,
-    MAP_GUN_TYPE_3_BUTTON_YELLOW,
-    MAP_GUN_TYPE_4_BUTTON_YELLOW,
-    CONSOLE_IDENTIFY_BUTTON_GREEN,
-    CONSOLE_IDENTIFY_BUTTON_RED,
-    CONSOLE_IDENTIFY_BUTTON_YELLOW,
     ITEM_BROWSER_LEFT_BUTTON,
     ITEM_BROWSER_RIGHT_BUTTON,
     ITEM_BROWSER_EXIT_BUTTON,
@@ -527,7 +475,9 @@ enum
     LEFT_TUX_SHOP_BUTTON,
     RIGHT_TUX_SHOP_BUTTON,
     LEFT_LEVEL_EDITOR_BUTTON,
+    LEFT_LEVEL_EDITOR_BUTTON_PUSHED,
     RIGHT_LEVEL_EDITOR_BUTTON,
+    RIGHT_LEVEL_EDITOR_BUTTON_PUSHED,
 
     NUMBER_SELECTOR_OK_BUTTON,
     NUMBER_SELECTOR_LEFT_BUTTON,
@@ -535,55 +485,87 @@ enum
 
     BUY_BUTTON,
     SELL_BUTTON,
-    TAKE_BUTTON,
-    PUT_BUTTON,
     REPAIR_BUTTON,
     IDENTIFY_BUTTON,
 
     OPEN_CLOSE_SKILL_EXPLANATION_BUTTON,
 
     GO_LEVEL_NORTH_BUTTON,
+    GO_LEVEL_NORTH_BUTTON_PUSHED,
     GO_LEVEL_SOUTH_BUTTON,
+    GO_LEVEL_SOUTH_BUTTON_PUSHED,
     GO_LEVEL_EAST_BUTTON,
+    GO_LEVEL_EAST_BUTTON_PUSHED,
     GO_LEVEL_WEST_BUTTON,
+    GO_LEVEL_WEST_BUTTON_PUSHED,
     EXPORT_THIS_LEVEL_BUTTON,
+    EXPORT_THIS_LEVEL_BUTTON_PUSHED,
     LEVEL_EDITOR_SAVE_SHIP_BUTTON,
+    LEVEL_EDITOR_SAVE_SHIP_BUTTON_PUSHED,
+    LEVEL_EDITOR_SAVE_SHIP_BUTTON_OFF,
+    LEVEL_EDITOR_SAVE_SHIP_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON,
+    LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON,
+    LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_CONNECTION_RED_BUTTON,
+    LEVEL_EDITOR_TOGGLE_CONNECTION_RED_BUTTON_PUSHED,
 
     LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON,
+    LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON_PUSHED,
     LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON,
+    LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON_PUSHED,
 
     LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON,
+    LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON_PUSHED,
     LEVEL_EDITOR_ZOOM_IN_BUTTON,
+    LEVEL_EDITOR_ZOOM_IN_BUTTON_PUSHED,
     LEVEL_EDITOR_ZOOM_OUT_BUTTON,
+    LEVEL_EDITOR_ZOOM_OUT_BUTTON_PUSHED,
     LEVEL_EDITOR_RECURSIVE_FILL_BUTTON,
+    LEVEL_EDITOR_RECURSIVE_FILL_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON,
+    LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_OBSTACLE_DESCRIPTION_BUTTON,
+    LEVEL_EDITOR_NEW_OBSTACLE_DESCRIPTION_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON,
+    LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_ITEM_BUTTON,
+    LEVEL_EDITOR_NEW_ITEM_BUTTON_PUSHED,
     LEVEL_EDITOR_ESC_BUTTON,
+    LEVEL_EDITOR_ESC_BUTTON_PUSHED,
     LEVEL_EDITOR_LEVEL_RESIZE_BUTTON,
-    LEVEL_EDITOR_KEYMAP_BUTTON,
+    LEVEL_EDITOR_LEVEL_RESIZE_BUTTON_PUSHED,
     LEVEL_EDITOR_QUIT_BUTTON,
+    LEVEL_EDITOR_QUIT_BUTTON_PUSHED,
 
     LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON,
+    LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON_PUSHED,
     LEVEL_EDITOR_UNDERGROUND_LIGHT_OFF_BUTTON,
+    LEVEL_EDITOR_UNDERGROUND_LIGHT_OFF_BUTTON_PUSHED,
 
-    LEVEL_EDITOR_TOGGLE_TUX_BUTTON,
-    LEVEL_EDITOR_TOGGLE_TUX_BUTTON_OFF,
     LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON,
+    LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON,
+    LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON,
+    LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_OFF,
-    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON ,
+    LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_OFF_PUSHED,
+    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON,
+    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON_OFF,
-    LEVEL_EDITOR_TOGGLE_GRID_BUTTON,
-    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL,
+    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_GRID_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_OFF_PUSHED,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_PUSHED,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL_PUSHED,
 
     LEVEL_EDITOR_NEXT_ITEM_GROUP_BUTTON,
     LEVEL_EDITOR_PREV_ITEM_GROUP_BUTTON,
@@ -592,6 +574,10 @@ enum
     LEVEL_EDITOR_NEXT_SUFFIX_BUTTON,
     LEVEL_EDITOR_PREV_SUFFIX_BUTTON,
     LEVEL_EDITOR_CANCEL_ITEM_DROP_BUTTON,
+    LEVEL_EDITOR_UNDO_BUTTON,
+    LEVEL_EDITOR_UNDO_BUTTON_PUSHED,
+    LEVEL_EDITOR_REDO_BUTTON,
+    LEVEL_EDITOR_REDO_BUTTON_PUSHED,
 
     SAVE_GAME_BANNER,
     LOAD_GAME_BANNER,
@@ -599,8 +585,6 @@ enum
     WEAPON_RECT_BUTTON,
     DRIVE_RECT_BUTTON,
     SHIELD_RECT_BUTTON,
-    AUX1_RECT_BUTTON,
-    AUX2_RECT_BUTTON,
     HELMET_RECT_BUTTON,
     ARMOUR_RECT_BUTTON,
 
@@ -659,21 +643,18 @@ enum
     MAX_MOUSE_PRESS_BUTTONS
   };
 
-#define CHAT_SUBDIALOG_WINDOW_X ((260)*GameConfig . screen_width/640)
-#define CHAT_SUBDIALOG_WINDOW_Y ((42)*GameConfig . screen_height/480)
-#define CHAT_SUBDIALOG_WINDOW_W ((337)*GameConfig . screen_width/640)
-#define CHAT_SUBDIALOG_WINDOW_H ((216)*GameConfig . screen_height/480)
+#define CHAT_SUBDIALOG_WINDOW_X ((260)*GameConfig . screen_width/640.0)
+#define CHAT_SUBDIALOG_WINDOW_Y ((42)*GameConfig . screen_height/480.0)
+#define CHAT_SUBDIALOG_WINDOW_W ((337)*GameConfig . screen_width/640.0)
+#define CHAT_SUBDIALOG_WINDOW_H ((216)*GameConfig . screen_height/480.0)
 
-#define STR_BASE_X 100
-#define STR_NOW_X 148
-#define STR_Y 143
-#define DEX_Y 171
-#define MAG_Y 197
-#define VIT_Y 227
-#define POINTS_Y 252
+#define STR_X 95
+#define STR_Y 140
+#define DEX_Y 165
+#define MAG_Y 190
+#define VIT_Y 215
+#define POINTS_Y 245
 
-#define BUTTON_MOD_X (-6)
-#define BUTTON_MOD_Y (-4)
 #define BUTTON_WIDTH 35
 #define BUTTON_HEIGHT 19
 
@@ -697,7 +678,7 @@ enum
 #define TUX_RUN_CYCLE_PHASES   10
 #define TUX_TOTAL_PHASES (TUX_SWING_PHASES+TUX_BREATHE_PHASES+TUX_GOT_HIT_PHASES+TUX_WALK_CYCLE_PHASES+TUX_RUN_CYCLE_PHASES)
 
-#define TUX_RUNNING_SPEED (7.1)
+#define TUX_RUNNING_SPEED (5.5)
 #define TUX_WALKING_SPEED (2.5)
 
 #define TUX_MODELS              13
@@ -708,17 +689,20 @@ enum
 #define WAIT_AFTER_KILLED	3.0  // time to wait and still display pictures after the destruction of 
                                      // the players droid.  This is now measured in seconds and can be a float
 #define WAIT_AFTER_GAME_WON	5.0  // how long to show tux after the game is won
-#define WAIT_COLLISION		2.5 // after a little collision with Tux or another enemy, hold position for a while
+#define WAIT_COLLISION		1 // after a little collision with Tux or another enemy, hold position for a while
                                     // this variable describes the amount of time in SECONDS 
 #define FLASH_DURATION_IN_SECONDS 0.1
 
 #define ALLBLASTTYPES		3	/* number of different exposions */
 
+#define MAX_MELEE_SHOTS		100
 #define MAXBULLETS		100	/* maximum possible Bullets in the air */
 #define MAXBLASTS		100	/* max. possible Blasts visible */
 #define AGGRESSIONMAX  		100
 #define ROBOT_MAX_WAIT_BETWEEN_SHOTS 5  // how long shoud each droid wait at most until
                                         // is considers fireing again?
+#define ATTACK_MOVE_RATE 0.2 // Minimum number of seconds between to displacement computation
+                             // during attack
 
 //--------------------
 // Map- and levelrelated defines
@@ -727,10 +711,6 @@ enum
 
 #define MAX_WP_CONNECTIONS 	12
 #define MAX_MAP_LINES 		100
-#define MAX_STATEMENTS_PER_LEVEL 30
-#define MAX_CLEARANCES 100
-#define MAX_PASSWORDS 100
-#define MAX_PASSWORD_LENGTH 20
 #define MAX_COOKIES 100
 #define MAX_COOKIE_LENGTH 200
 #define MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX 100
@@ -743,37 +723,134 @@ enum
 
 #define MAX_SAVED_CHARACTERS_ON_DISK 7
 
-#define MAX_ENEMYS_ON_SHIP	1000
-#define MAX_INFLU_POSITION_HISTORY 10000
-#define MAX_MISSIONS_IN_GAME 10         // how many missions can there be at most in Freedroid
-#define MAX_MISSION_TRIGGERED_ACTIONS 10
+#define MAX_INFLU_POSITION_HISTORY 500
+#define MAX_MISSIONS_IN_GAME 17         // how many missions can there be at most in Freedroid
 #define MAX_MISSION_DESCRIPTION_TEXTS 10
 
-#define MAX_LEVELS		39   	// how many map levels are allowed in one ship
+#define MAX_LEVELS		50  	// how many map levels are allowed in one ship
 
-#define MAX_EVENT_TRIGGERS      100   // how many event triggers at most to allow
-#define MAX_TRIGGERED_ACTIONS_IN_GAME   100   // how many triggerable actions to allow at most
+#define MAX_EVENT_TRIGGERS      500   // how many event triggers at most to allow
 
-#define MAXWAYPOINTS 		200
-#define MAX_DOORS_ON_LEVEL 	120
+#define MAXWAYPOINTS 		300
+#define MAX_DOORS_ON_LEVEL 	150
 #define MAX_AUTOGUNS_ON_LEVEL 	60
 #define MAX_REFRESHES_ON_LEVEL	39
 #define MAX_TELEPORTERS_ON_LEVEL 20
 
 #define MAX_PHASES_IN_A_BULLET 12
 #define PHASES_OF_EACH_BLAST 20
-#define MAX_STEPS_IN_GIVEN_COURSE 100
-
-#undef USE_MISS_HIT_ARRAYS
-#define UNCHECKED 0
-#define HIT 1
-#define MISS 2
 
 #define ITEM_INVENTORY_IMAGE_FILE_NAME "File or directory name for inventory image=\""
 #define ITEM_DROP_SOUND_FILE_NAME "Item uses drop sound with filename=\""
 
-#define UNIVERSAL_COORD_W(W) (int)((float)W * ((float)(GameConfig . screen_width) / 640.0))
-#define UNIVERSAL_COORD_H(H) (int)((float)H * ((float)(GameConfig . screen_height) / 480.0))
+#define UNIVERSAL_COORD_W(W) (int)((float)(W) * ((float)(GameConfig . screen_width) / 640.0))
+#define UNIVERSAL_COORD_H(H) (int)((float)(H) * ((float)(GameConfig . screen_height) / 480.0))
+
+//--------------------
+// And now we put here all the rectangle definitions and that...
+//
+
+// Game message window
+#define SUBTITLEW_RECT_X 120
+#define SUBTITLEW_RECT_Y 389
+#define SUBTITLEW_RECT_W 415
+#define SUBTITLEW_RECT_H 44
+
+// the inventory rect is the rectangle inside the inventory screen,
+// where all the items "in the backpack" including the quick inventory
+// are displayed.  it's relative to the top left corner, as always.
+//
+
+#define INVENTORY_RECT_Y 260
+#define INVENTORY_RECT_X 8
+
+#define CURRENT_SKILL_RECT_X 537
+#define CURRENT_SKILL_RECT_Y (GameConfig . screen_height - 80 * GameConfig . screen_height / 480)
+#define CURRENT_SKILL_RECT_W 64
+#define CURRENT_SKILL_RECT_H 64
+
+// 388 is the maximum y size to fit items up to 6 inventory squares high
+// 398 is the maximum y size to fit items up to 5 inventory squares high
+#define CURRENT_WEAPON_RECT_X 35
+#define CURRENT_WEAPON_RECT_Y 398
+#define CURRENT_WEAPON_RECT_W 64
+#define CURRENT_WEAPON_RECT_H 64
+
+#define WHOLE_HEALTH_RECT_Y (480-85)
+#define WHOLE_HEALTH_RECT_W 6
+#define WHOLE_HEALTH_RECT_H 50
+#define WHOLE_HEALTH_RECT_X 614
+#define HEALTH_RECT_COLOR (63*64*64 + 64*8)
+
+#define GOLDEN_SCHNITT ((sqrt(5)-1)/2)
+
+#define WHOLE_RUNNING_POWER_RECT_X 6
+#define WHOLE_RUNNING_POWER_RECT_Y (480-85)
+#define WHOLE_RUNNING_POWER_RECT_W 6
+#define WHOLE_RUNNING_POWER_RECT_H 50
+
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_X 19
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_Y WHOLE_RUNNING_POWER_RECT_Y
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_W WHOLE_RUNNING_POWER_RECT_W
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_H WHOLE_RUNNING_POWER_RECT_H
+
+#define WHOLE_FORCE_RECT_Y ( WHOLE_HEALTH_RECT_Y )
+#define WHOLE_FORCE_RECT_W ( WHOLE_HEALTH_RECT_W )
+#define WHOLE_FORCE_RECT_H ( WHOLE_HEALTH_RECT_H )
+#define WHOLE_FORCE_RECT_X ( 640-12)
+#define FORCE_RECT_COLOR ( 0 * 64 * 64 + 64 * 0 + 63 )
+
+//--------------------
+// The weapon rectangle is of course for the weapon the Tux is
+// using.  When using a 2-handed weapon, the 'shield' rectangle
+// will ALSO be used to show this weapon, since the shield-hand
+// is also needed to wield such a thing.
+//
+// 31 is the minimum x size to fit items up to 4 inventory squares wide
+#define WEAPON_RECT_WIDTH 67
+#define WEAPON_RECT_HEIGHT 112
+#define WEAPON_RECT_X 31
+#define WEAPON_RECT_Y 117
+
+//--------------------
+// The armour rectangle is where you can place some armour
+// for the body of the Tux.
+//
+#define ARMOUR_RECT_WIDTH 66
+#define ARMOUR_RECT_HEIGHT 92
+#define ARMOUR_RECT_X 130
+#define ARMOUR_RECT_Y 70
+
+//--------------------
+// The shield rectangle is obviously for the shields, but not only
+// that:  when using a 2-handed weapon, its also good for the weapon
+// again, to indicate, that no shield can be used in conjunction with
+// such a 2-handed weapon.
+//
+#define SHIELD_RECT_WIDTH 70
+#define SHIELD_RECT_HEIGHT 112
+#define SHIELD_RECT_X 230
+#define SHIELD_RECT_Y 116
+
+//--------------------
+// The 'drive' rectangle is actually for the footwear.
+//
+#define DRIVE_RECT_WIDTH 64
+#define DRIVE_RECT_HEIGHT 64
+#define DRIVE_RECT_X 130
+#define DRIVE_RECT_Y 175
+
+//--------------------
+// Now this is the 'helmet' rectangle.
+//
+#define HELMET_RECT_WIDTH 64
+#define HELMET_RECT_HEIGHT 64
+#define HELMET_RECT_X 25
+#define HELMET_RECT_Y 22
+
+// Flags for thrown items. see PutItem function in view.c
+#define PUT_ONLY_THROWN_ITEMS 3
+#define PUT_NO_THROWN_ITEMS 4
 
 
 //--------------------
@@ -824,19 +901,6 @@ enum _explosions
 enum _status
 {
     INFOUT=-30,
-    MOBILE,
-    TRANSFERMODE,
-    WEAPON,
-    CAPTURED,
-    COMPLETE,
-    REJECTED,
-    CONSOLE,
-    DEBRIEFING,
-    PAUSE,
-    CHEESE,
-    ELEVATOR,
-    BRIEFING,
-    MENU
 };
 
 enum _busytype
@@ -844,8 +908,18 @@ enum _busytype
     NONE=0,
     DRINKING_POTION=1,
     WEAPON_FIREWAIT,
-    WEAPON_RELOAD
+    WEAPON_RELOAD,
+    THROWING_GRENADE,
+    RUNNING_PROGRAM,
 };
+
+enum _attackhit
+{
+    ATTACK_HIT_BOTS=1,
+    ATTACK_HIT_HUMANS=2,
+    ATTACK_HIT_ALL=3,
+};
+
 //--------------------
 // The possible edito modes for the level editor
 //
@@ -858,30 +932,42 @@ enum _level_editor_edit_modes
     LEVEL_EDITOR_SELECTION_CONTAINERS = 4,
     LEVEL_EDITOR_SELECTION_PLANTS = 5,
     LEVEL_EDITOR_SELECTION_ALL = 6,
-    NUMBER_OF_LEVEL_EDITOR_GROUPS = 7
+    LEVEL_EDITOR_SELECTION_QUICK = 7, 
+    NUMBER_OF_LEVEL_EDITOR_GROUPS = 8
 };
 
+// possible directions for a line of walls
+// We use 1 et -1, because north is the opposite of south.
+enum _level_editor_directions
+{
+    UNDEFINED = 0,
+    NORTH = 1,
+    EAST = 2,
+    SOUTH = -1,
+    WEST = -2
+};
+
+
 enum _enemy_combat_state_machine_states
 {
-    MOVE_ALONG_RANDOM_WAYPOINTS = -301 , 
-    STOP_AND_EYE_TUX = -302 ,
-    MAKE_ATTACK_RUN = -303 , 
-    HUNTING_TUX = -304 , 
-    SEEK_BETTER_POSITION = -305 ,
-    FIGHT_ON_TUX_SIDE = -306 ,
-    RELENTLESS_FIRE_TO_GIVEN_POSITION = -307 ,
-    TURN_THOWARDS_NEXT_WAYPOINT = -308 ,
-    RUSH_TUX_ON_SIGHT_AND_OPEN_TALK = -309 ,
-    WAIT_AND_TURN_AROUND_AIMLESSLY = -310 ,
-    WAYPOINTLESS_WANDERING = -311 ,
-    TURN_THOWARDS_WAYPOINTLESS_SPOT = -312 
-    // BACK_OFF_AFTER_GETTING_HIT, 
+    MOVE_ALONG_RANDOM_WAYPOINTS = 0, 
+    STOP_AND_EYE_TARGET = 1,
+    ATTACK = 2, 
+    PARALYZED = 3,
+    COMPLETELY_FIXED = 4,
+    FOLLOW_TUX = 5,
+    RETURNING_HOME = 6,
+    SELECT_NEW_WAYPOINT = 7,
+    TURN_TOWARDS_NEXT_WAYPOINT = 8,
+    RUSH_TUX_AND_OPEN_TALK = 9,
+    WAYPOINTLESS_WANDERING = 10,
+    UNDEFINED_STATE = 100
 };
 enum _enemy_combat_target_codes
 {
-    ATTACK_TARGET_IS_ENEMY = -201,
-    ATTACK_TARGET_IS_PLAYER = -202,
-    ATTACK_TARGET_IS_NOTHING = -203
+    ATTACK_TARGET_IS_ENEMY = 101,
+    ATTACK_TARGET_IS_PLAYER = 102,
+    ATTACK_TARGET_IS_NOTHING = 103
 };
 enum
 {
@@ -889,23 +975,12 @@ enum
 }; 
 enum
 {
-    TRANSPARENCY_FOR_WALLS = -501 ,
-    TRANSPARENCY_FOR_SEE_THROUGH_OBJECTS = -502
+    TRANSPARENCY_NONE = 0,
+    TRANSPARENCY_FOR_WALLS = 10 ,
+    TRANSPARENCY_FOR_SEE_THROUGH_OBJECTS = 11,
+    TRANSPARENCY_CUROBJECT = 12
 }; 
 
-
-//--------------------
-// The possible networking statuses of clients.
-//
-enum _networking_status
-{
-    NETWORK_ERROR = 0 ,
-    UNCONNECTED , 
-    CONNECTION_FRESHLY_OPENED ,
-    NAME_HAS_BEEN_TRANSMITTED , 
-    GAME_ON
-};
-
 //--------------------
 // It is possible, that the Tux must fist move somewhere AND
 // then open a chest or move somewhere AND then pick something up
@@ -918,7 +993,6 @@ enum _combo_action_types
     COMBO_ACTION_OPEN_CHEST = 15 ,
     COMBO_ACTION_PICK_UP_ITEM = 16 ,
     COMBO_ACTION_SMASH_BARREL = 17 ,
-    COMBO_ACTION_LOOT_OBSTACLE = 18 ,
 };
 
 //--------------------
@@ -929,17 +1003,7 @@ enum
   {
     DO_NOTHING = -1 , 
     BUY_1_ITEM = 1 , 
-    BUY_10_ITEMS = 2 , 
-    BUY_100_ITEMS = 3 , 
     SELL_1_ITEM = 4 , 
-    SELL_10_ITEMS = 5 , 
-    SELL_100_ITEMS = 6 , 
-    PUT_1_ITEM = 6 , 
-    PUT_10_ITEMS = 7 , 
-    PUT_100_ITEMS = 8 , 
-    TAKE_1_ITEM = 9 , 
-    TAKE_10_ITEMS = 10 , 
-    TAKE_100_ITEMS = 11 ,
     REPAIR_ITEM = 12 , 
     IDENTIFY_ITEM = 13 
   };
@@ -1411,6 +1475,60 @@ enum
   ISO_DIS_AUTOGUN_E,
   ISO_DIS_AUTOGUN_S,
 
+  ISO_BRICK_WALL_CABLES_H ,
+  ISO_BRICK_WALL_CABLES_V ,
+
+  ISO_BRICK_WALL_CABLES_CORNER_1 ,
+  ISO_BRICK_WALL_CABLES_CORNER_2 , 
+  ISO_BRICK_WALL_CABLES_CORNER_3 , 
+  ISO_BRICK_WALL_CABLES_CORNER_4 ,
+
+  ISO_RESTAURANT_DESK_1,
+  ISO_RESTAURANT_DESK_2,
+  ISO_RESTAURANT_BIGSHELF_1,
+  ISO_RESTAURANT_BIGSHELF_2,
+
+  ISO_CRYSTALS_1,
+  ISO_CRYSTALS_2,
+  ISO_CRYSTALS_3,
+  ISO_CRYSTALS_4,
+  ISO_CRYSTALS_5,
+  ISO_CRYSTALS_6,
+
+  ISO_GREY_WALL_CORNER_1,
+  ISO_GREY_WALL_CORNER_2,
+  ISO_GREY_WALL_CORNER_3,
+  ISO_GREY_WALL_CORNER_4, // = 347
+  
+  ISO_BROKEN_GLASS_WALL_1,
+  ISO_OUTER_DOOR_V_OFFLINE ,
+  ISO_OUTER_DOOR_H_OFFLINE ,
+  ISO_DH_DOOR_LOCKED ,
+  ISO_DV_DOOR_LOCKED ,
+
+  ISO_DH_DOOR_000_OPEN ,
+  ISO_DH_DOOR_025_OPEN ,
+  ISO_DH_DOOR_050_OPEN ,
+  ISO_DH_DOOR_075_OPEN ,
+  ISO_DH_DOOR_100_OPEN ,
+
+  ISO_DV_DOOR_000_OPEN ,
+  ISO_DV_DOOR_025_OPEN ,
+  ISO_DV_DOOR_050_OPEN ,
+  ISO_DV_DOOR_075_OPEN ,
+  ISO_DV_DOOR_100_OPEN , // = 362
+
+  ISO_BASIN_1,
+  ISO_BASIN_2,
+  ISO_BASIN_3,
+  ISO_BASIN_4,
+
+  ISO_DESKCHAIR_1,
+  ISO_DESKCHAIR_2,
+  ISO_DESKCHAIR_3,
+
+  ISO_EXIT_5,
+  ISO_EXIT_6,
   NUMBER_OF_OBSTACLE_TYPES
 };
 
@@ -1509,5 +1627,19 @@ enum
   ALL_ISOMETRIC_FLOOR_TILES 
 };
 
+#define ITEM_TAKE_DIST (1.2)
+// Maximum and minimum distance for a melee fight
+#define SQUARED_MELEE_MAX_DIST (2.25)
+#define MELEE_MIN_DIST (1.0)
+#define SQUARED_MELEE_MIN_DIST (MELEE_MIN_DIST*MELEE_MIN_DIST)
+// Distance at which a bot with a melee weapon will start to find 
+// a free place around its target 
+#define SQUARED_MELEE_APPROACH_DIST (16.0)
+// Minimum distance to shoot with a range weapon
+#define SQUARED_RANGE_SHOOT_MIN_DIST (7.0)
+  
+//-----
+// Pathfinder constants
+#define DEBUG_TUX_PATHFINDING 1  // debug level for tux pathfinding...
 
 #endif
