@@ -1,5 +1,6 @@
 /* 
  *
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
  *
@@ -22,10 +23,10 @@
  *  MA  02111-1307  USA
  *
  */
-/* ----------------------------------------------------------------------
+/**
  * This file does everything that has to do with the takeover game from
  * the original paradroid game.
- * ---------------------------------------------------------------------- */
+ */
 
 #define _takeover_c
 
@@ -38,6 +39,7 @@
 #include "takeover.h"
 #include "map.h"
 
+enemy * cDroid; 
 Uint32 cur_time;  		// current time in ms 
 SDL_Surface *to_blocks;         // the global surface containing all game-blocks 
 
@@ -120,7 +122,7 @@ int LeaderColor = GELB;		/* momentary leading color */
 int YourColor = GELB;
 int OpponentColor = VIOLETT;
 int OpponentType;		/* The druid-type of your opponent */
-int DroidNum;
+enemy * cdroid;
 
 /* the display  column */
 int DisplayColumn[NUM_LINES] = {
@@ -138,6 +140,157 @@ void EvaluatePlayground ( void );
 float EvaluatePosition ( int color , int row , int layer );
 void AdvancedEnemyTakeoverMovements (void);
 
+/** 
+ * Display the picture of a droid
+ */
+static void ShowDroidPicture (int PosX, int PosY, int Number )
+{
+    SDL_Surface *tmp;
+    SDL_Rect target;
+    char ConstructedFileName[5000];
+char fpath[2048];
+    static char LastImageSeriesPrefix[1000] = "NONE_AT_ALL";
+#define NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION 32
+    static SDL_Surface *DroidRotationSurfaces[ NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION ] = { NULL } ;
+    SDL_Surface *Whole_Image;
+    int i;
+    int RotationIndex;
+    
+    DebugPrintf ( 2 , "\nvoid ShowDroidPicture(...): Function call confirmed.");
+    
+    if ( !strcmp ( Druidmap[ Number ] . droid_portrait_rotation_series_prefix , "NONE_AVAILABLE_YET" ) )
+	return; // later this should be a default-correction instead
+    
+    //--------------------
+    // Maybe we have to reload the whole image series
+    //
+    if ( strcmp ( LastImageSeriesPrefix , Druidmap [ Number ] . droid_portrait_rotation_series_prefix ) )
+    {
+	//--------------------
+	// Maybe we have to free the series from an old item display first
+	//
+	if ( DroidRotationSurfaces[ 0 ] != NULL )
+	{
+	    for ( i = 1 ; i < NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION ; i ++ )
+	    {
+		SDL_FreeSurface ( DroidRotationSurfaces[ i ] ) ;
+	    }
+	}
+	
+	//--------------------
+	// Now we can start to load the whole series into memory
+	//
+	for ( i=0 ; i < NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION ; i++ )
+	{
+	    if ( !strcmp ( Druidmap[ Number ] . droid_portrait_rotation_series_prefix , "NONE_AVAILABLE_YET" ) )
+	    {
+		Terminate ( ERR );
+	    }
+	    else
+	    {
+		sprintf ( ConstructedFileName , "droids/%s/portrait_%04d.jpg" , Druidmap[ Number ] . droid_portrait_rotation_series_prefix , i+1 );
+		DebugPrintf ( 1 , "\nConstructedFileName = %s " , ConstructedFileName );
+	    }
+	    
+	    // We must remember, that his is already loaded of course
+	    strcpy ( LastImageSeriesPrefix , Druidmap [ Number ] . droid_portrait_rotation_series_prefix );
+	    
+	    find_file (ConstructedFileName , GRAPHICS_DIR, fpath, 0 );
+	    
+	    Whole_Image = our_IMG_load_wrapper( fpath ); // This is a surface with alpha channel, since the picture is one of this type
+	    if ( Whole_Image == NULL )
+	    {
+		fprintf( stderr, "\n\nfpath: %s. \n" , fpath );
+		ErrorMessage ( __FUNCTION__  , "\
+Freedroid was unable to load an image of a rotated droid into memory.\n\
+This error indicates some installation problem with freedroid.",
+					   PLEASE_INFORM, IS_FATAL );
+	    }
+	    
+	    SDL_SetAlpha( Whole_Image , 0 , SDL_ALPHA_OPAQUE );
+	    
+	    DroidRotationSurfaces[i] = our_SDL_display_format_wrapperAlpha( Whole_Image ); // now we have an alpha-surf of right size
+	    SDL_SetColorKey( DroidRotationSurfaces[i] , 0 , 0 ); // this should clear any color key in the dest surface
+	    
+	    SDL_FreeSurface( Whole_Image );
+	}
+    }
+
+    RotationIndex = ( SDL_GetTicks() / 50 ) ;
+    
+    RotationIndex = RotationIndex - ( RotationIndex / NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION ) * NUMBER_OF_IMAGES_IN_DROID_PORTRAIT_ROTATION ;
+    
+    tmp = DroidRotationSurfaces[ RotationIndex ] ;
+    
+    SDL_SetClipRect( Screen , NULL );
+    Set_Rect ( target, PosX, PosY, GameConfig . screen_width, GameConfig . screen_height);
+    our_SDL_blit_surface_wrapper( tmp , NULL, Screen , &target);
+    
+    DebugPrintf ( 2 , "\nvoid ShowDroidPicture(...): Usual end of function reached.");
+    
+}; // void ShowDroidPicture ( ... )
+
+/* ------------------------------------------------------------
+ * display infopage page of droidtype
+ * does update the screen, no our_SDL_flip_wrapper() necesary !
+ * ------------------------------------------------------------ */
+static void ShowDroidInfo ( int droidtype, int Displacement , char ShowArrows )
+{
+    char *item_name;
+    int type;
+    char InfoText[10000];
+    char TextChunk[2000];
+    
+    //--------------------
+    // We initialize the text rectangle
+    //
+    Cons_Text_Rect . x = 258 * GameConfig . screen_width / 640 ; 
+    Cons_Text_Rect . y = 89 * GameConfig . screen_height / 480 ; 
+    Cons_Text_Rect . w = 346 * GameConfig . screen_width / 640 ; 
+    Cons_Text_Rect . h = 282 * GameConfig . screen_height / 480 ;
+    
+    SDL_SetClipRect ( Screen , NULL );
+    
+    blit_special_background ( ITEM_BROWSER_BG_PIC_BACKGROUND_CODE ) ;
+    
+    ShowDroidPicture ( 45 * GameConfig . screen_width / 640 , 190 * GameConfig . screen_height / 480 , droidtype );
+    
+    //--------------------
+    // We fill out the header area of the items browser.
+    //
+    SetCurrentFont ( Menu_BFont );
+    strcpy ( TextChunk , Druidmap [ droidtype ] . druidname );
+    CutDownStringToMaximalSize ( TextChunk , 225 );
+    PutString ( Screen , 330 * GameConfig . screen_width / 640 , 38 * GameConfig . screen_height / 480 , TextChunk );
+    
+    sprintf( InfoText, _("\
+Unit type %s\n\
+Entry : %d\n"), Druidmap[droidtype].druidname, droidtype + 1);
+    
+    if ( (type = Druidmap[droidtype].weapon_item.type) >= 0) // make sure item=-1 
+	item_name = D_(ItemMap[type].item_name);                 // does not segfault 
+    else 
+	item_name = _("none");
+    
+    sprintf( TextChunk , _("\nArmament : %s\n"),
+	     item_name);
+    strcat ( InfoText , TextChunk );
+    
+    sprintf ( TextChunk , _("\nNotes: %s\n"), D_(Druidmap[droidtype].notes));
+    strcat ( InfoText , TextChunk );
+    
+    SetCurrentFont( FPS_Display_BFont );
+    DisplayText (InfoText, Cons_Text_Rect.x, Cons_Text_Rect.y + Displacement , &Cons_Text_Rect , TEXT_STRETCH );
+    
+    if ( ShowArrows ) 
+    {
+	ShowGenericButtonFromList ( UP_BUTTON );
+	ShowGenericButtonFromList ( DOWN_BUTTON );
+    }
+    
+}; // void ShowDroidInfo ( ... )
+
+
 /*-----------------------------------------------------------------
  *
  * This function manages the whole takeover game of Tux against 
@@ -148,24 +301,26 @@ void AdvancedEnemyTakeoverMovements (void);
  *
  *-----------------------------------------------------------------*/
 int
-Takeover ( int enemynum )
+Takeover ( enemy * target )
 {
     int row;
     int FinishTakeover = FALSE;
     char *message;
-    int ClearanceIndex;
     int Finished = FALSE;
-    int WasPressed = FALSE ;
     int Displacement = 0 ;
     int reward = 0;
-    char game_message_text [ 5000 ] ;
+    char game_message_text [ 500 ] ;
+    SDL_Event event;
+    int old_status;
+
+    old_status = game_status;
 
     //--------------------
     // Prevent distortion of framerate by the delay coming from 
     // the time spent in the menu.
     //
     Activate_Conservative_Frame_Computation ();
-    
+
     //--------------------
     // We set the UserRect to full again, no matter what other windows might
     // be open right now...
@@ -174,94 +329,100 @@ Takeover ( int enemynum )
     User_Rect . y = 0 ;
     User_Rect . w = GameConfig . screen_width ;
     User_Rect . h = GameConfig . screen_height ;
-    
+
     //--------------------
     // Maybe takeover graphics haven't been loaded yet.  Then we do this
     // here now and for once.  Later calls will be ignored inside the function.
     //
     GetTakeoverGraphics ( ) ;
-    
+
     while (SpacePressed () || MouseLeftPressed()) ;  // make sure space is release before proceed 
-    
+
     SwitchBackgroundMusicTo ( TAKEOVER_BACKGROUND_MUSIC_SOUND ); // now this is a STRING!!!
-    
+
     DisplayBanner ( ) ;
-    Me [ 0 ] . status = MOBILE; // the new status _after_ the takeover game 
-    
+
     //--------------------
     // Now it is time to display the enemy of this whole takeover process...
     //
     if ( GameConfig . auto_display_to_help ) 
 	{
-        PlayATitleFile ( "TakeoverInstructions.title" );
+	PlayATitleFile ( "TakeoverInstructions.title" );
 	GameConfig . auto_display_to_help = 0;
 	}
 
-    while ( !Finished )
-    {
-	ShowDroidInfo ( AllEnemys[enemynum].type, Displacement , TRUE );
+    while ( !Finished )	{
+	ShowDroidInfo ( target->type, Displacement , TRUE );
 	ShowGenericButtonFromList ( TAKEOVER_HELP_BUTTON ) ;
 	blit_our_own_mouse_cursor ( );
-	our_SDL_flip_wrapper ( Screen );
-	
-	if ( MouseCursorIsOnButton( UP_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && MouseLeftPressed() && !WasPressed )
-	{
-	    MoveMenuPositionSound();
-	    Displacement += FontHeight ( GetCurrentFont () );
+	our_SDL_flip_wrapper();
+
+	while(SDL_PollEvent(&event)) {
+
+	    if (event.type == SDL_QUIT) {
+		Terminate(0);
+	    }
+
+	    if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT) {
+		if ( MouseCursorIsOnButton( UP_BUTTON , GetMousePos_x()  , GetMousePos_y()  ))
+		    {
+		    MoveMenuPositionSound();
+		    Displacement += FontHeight ( GetCurrentFont () );
+		    }
+		else if ( MouseCursorIsOnButton( DOWN_BUTTON , GetMousePos_x()  , GetMousePos_y()  ))
+		    {
+		    MoveMenuPositionSound();
+		    Displacement -= FontHeight ( GetCurrentFont () );
+		    }
+		else if ( MouseCursorIsOnButton( DRUID_SHOW_EXIT_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  )) 
+		    {
+		    Finished = TRUE;
+		    }
+		else if ( MouseCursorIsOnButton( TAKEOVER_HELP_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  )) 
+		    {
+		    PlayATitleFile ( "TakeoverInstructions.title" );
+		    }
+	    } else if (event.type == SDL_KEYDOWN && ((event.key.keysym.sym == SDLK_SPACE) || (event.key.keysym.sym == SDLK_ESCAPE))) {
+		Finished = TRUE ;
+	    }	
 	}
-	else if ( MouseCursorIsOnButton( DOWN_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && MouseLeftPressed() && !WasPressed )
-	{
-	    MoveMenuPositionSound();
-	    Displacement -= FontHeight ( GetCurrentFont () );
-	}
-	else if ( MouseCursorIsOnButton( DRUID_SHOW_EXIT_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  ) && MouseLeftPressed() && !WasPressed ) 
-	{
-	    Finished = TRUE;
-	}
-	else if ( MouseCursorIsOnButton( TAKEOVER_HELP_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  ) && MouseLeftPressed() && !WasPressed ) 
-	{
-	    PlayATitleFile ( "TakeoverInstructions.title" );
-	}
-	WasPressed = MouseLeftPressed(); 
-	
-	if ( SpacePressed() && !MouseLeftPressed() ) Finished = TRUE ;
-	
+	SDL_Delay(1);
     }
-    
+
     while ( !( !SpacePressed() && !EscapePressed() && !MouseLeftPressed() )) ;
-    
-    
+
+
     while (!FinishTakeover)
-    {
+	{
 	//--------------------
 	// Init Color-column and Capsule-Number for each opponenet and your color 
 	//
 	for (row = 0; row < NUM_LINES; row++)
-	{
+	    {
 	    DisplayColumn[row] = (row % 2);
 	    CapsuleCountdown[GELB][0][row] = -1;
 	    CapsuleCountdown[VIOLETT][0][row] = -1;
-	} // for row 
-	
+	    } // for row 
+
 	YourColor = GELB;
 	OpponentColor = VIOLETT;
-	
+
 	CapsuleCurRow[GELB] = 0;
 	CapsuleCurRow[VIOLETT] = 0;
-	
-	DroidNum = enemynum;
-	OpponentType = AllEnemys [ enemynum ] . type;
-	// NumCapsules[YOU] = 3 + ClassOfDruid (Me[0].type);
-	NumCapsules[YOU] = 3 + Me [ 0 ] . hacking_skill ;
+
+	OpponentType = target->type;
+	cDroid = target;
+	// NumCapsules[YOU] = 3 + ClassOfDruid (Me.type);
+	NumCapsules[YOU] = 2 + Me . base_skill_level [ get_program_index_with_name("Hacking") ];
 	NumCapsules[ENEMY] = 2 + Druidmap [ OpponentType ] . class ;
-	
+
 	InventPlayground ();
-	
+
 	EvaluatePlayground ();
-	
+
 	ShowPlayground ();
-	our_SDL_flip_wrapper (Screen);
-	
+	our_SDL_flip_wrapper();
+
 	ChooseColor ();
 
 	//--------------------
@@ -272,40 +433,37 @@ Takeover ( int enemynum )
 	// the game must be played again in the next loop...
 	//
 	PlayGame ();
-	
+
 	//--------------------
 	// We we evaluate the final score of the game.  Maybe we're done
 	// already, maybe not...
 	//
 	if ( LeaderColor == YourColor ) 
-	{
+	    {
 	    // SwitchBackgroundMusicTo (SILENCE);
 	    Takeover_Game_Won_Sound ();
-	    
+
 	    //--------------------
 	    // We allow to gain the current energy/full health that was still in the 
 	    // other droid, since all previous damage must be due to fighting damage,
 	    // and this is exactly the sort of damage can usually be cured in refreshes.
 	    //
-	    Me [ 0 ] . energy += AllEnemys [ enemynum ] . energy;
-	    Me [ 0 ] . health += Druidmap [ OpponentType ] . maxenergy;
-	    
+	    Me . energy += target->energy;
+
 	    //--------------------
 	    // We provide some security agains too high energy/health values gained
 	    // by very rapid successions of successful takeover attempts
 	    //
-	    if ( Me [ 0 ] . energy > Me [ 0 ] . maxenergy ) 
-		Me [ 0 ] . energy = Me [ 0 ] . maxenergy;
-	    if ( Me [ 0 ] . health > Me [ 0 ] . maxenergy ) 
-		Me [ 0 ] . health = Me [ 0 ] . maxenergy;
-	    
-	    Me [ 0 ] . type = AllEnemys [ enemynum ] . type;
-	    Me [ 0 ] . marker = AllEnemys [ enemynum ] . marker;
+	    if ( Me . energy > Me . maxenergy ) 
+		Me . energy = Me . maxenergy;
 
-	    reward = Druidmap [ AllEnemys [ enemynum ] . type ] . experience_reward * 3 ;
-	    Me [ 0 ] . Experience += reward;
+	    Me . type = target->type;
+	    Me . marker = target->marker;
+
+	    reward = Druidmap [ target->type ] . experience_reward * 1 ;
+	    Me . Experience += reward;
 	    sprintf ( game_message_text , _("For taking control of your enemy, you receive %d experience."),
-		      reward );
+		    reward );
 	    append_new_game_message ( game_message_text );
 
 	    //--------------------
@@ -315,35 +473,28 @@ Takeover ( int enemynum )
 	    // should be dropped after the successful takeover process, even if the
 	    // enemy isn't completely dead yet...
 	    //
-	    if ( AllEnemys [ enemynum ] . on_death_drop_item_code != (-1) )
-	    {
-		DropItemAt( AllEnemys [ enemynum ] . on_death_drop_item_code , 
-			    AllEnemys [ enemynum ] . pos . z ,
-			    AllEnemys [ enemynum ] . pos . x , 
-			    AllEnemys [ enemynum ] . pos . y , -1 , -1 , 2 , 1 );
-		AllEnemys [ enemynum ] . on_death_drop_item_code = -1;
-	    }  
+	    if ( target->on_death_drop_item_code != (-1) )
+		{
+		DropItemAt( target->on_death_drop_item_code , 
+			target->pos . z ,
+			target->pos . x , 
+			target->pos . y , -1 , -1 , 1 );
+		target->on_death_drop_item_code = -1;
+		}  
 
-	    AllEnemys [ enemynum ] . energy =  Druidmap [ AllEnemys [ enemynum ] . type ] . maxenergy ; 
+	    target->energy =  Druidmap [ target->type ] . maxenergy ; 
 
-	    //--------------------
-	    // After takeover, the enemy suddenly becomes friendly and will also have
-	    // a much nicer default 'combat state'...  the enemy can still start to
-	    // fight from that nicer state, maybe on the Tux side, this time :)
-	    //
-	    AllEnemys [ enemynum ] . is_friendly = TRUE ;
-	    AllEnemys [ enemynum ] . has_been_taken_over = TRUE ; 
-	    if ( AllEnemys [ enemynum ] .  stick_to_waypoint_system_by_default )
-		AllEnemys [ enemynum ] . combat_state = MOVE_ALONG_RANDOM_WAYPOINTS ;
-	    else
-		AllEnemys [ enemynum ] . combat_state = WAYPOINTLESS_WANDERING ;
+	    target->is_friendly = TRUE ;
+	    target->has_been_taken_over = TRUE ; 
+
+	    target->combat_state = WAYPOINTLESS_WANDERING ;
 
 	    //--------------------
 	    // When the bot is taken over, it should not turn hostile when
 	    // the rest of his former combat group (identified by having the
 	    // same marker) is attacked by the Tux.
 	    //
-	    AllEnemys [ enemynum ] . marker = 0 ;
+	    target->marker = 0 ;
 
 	    //--------------------
 	    // The takeover game earlier had some status banner, where the 
@@ -354,54 +505,58 @@ Takeover ( int enemynum )
 		message = _("You cheat");
 	    else				// won the proper way 
 		message = _("Complete");
-	    
+
 	    //--------------------
 	    // Ok.  The whole takeover is done now.  We can return...
 	    //
 	    FinishTakeover = TRUE;
-	}				// LeaderColor == YourColor 
+	    }				// LeaderColor == YourColor 
 	else if ( LeaderColor == OpponentColor )
-	{
+	    {
 	    Takeover_Game_Lost_Sound ();
 	    message = _("Rejected");
-	    Me [ 0 ] . energy *= 0.5 ;
-	    AllEnemys [ enemynum ] . energy = Druidmap [ AllEnemys [ enemynum ] . type ] . maxenergy ;
+	    Me . energy *= 0.5 ;
+	    target->energy = Druidmap [ target->type ] . maxenergy ;
 	    FinishTakeover = TRUE;
-	}			// if LeadColor == OpponentColor 
+	    }			// if LeadColor == OpponentColor 
 	else
-	{
+	    {
 	    Takeover_Game_Deadlock_Sound ();
 	    message = _("Deadlock");
-	}			// LeadColor == DRAW	
+	    }			// LeadColor == DRAW	
 
 	ShowPlayground ();
 	to_show_banner (message, NULL);
-	our_SDL_flip_wrapper (Screen);
-	
-    }	// while !FinishTakeover 
-    
+	our_SDL_flip_wrapper();
+	SDL_Delay(100);
+
+	}
+
     ClearGraphMem();
-    
-    SwitchBackgroundMusicTo ( CurLevel -> Background_Song_Name );
-    
+
+    SwitchBackgroundMusicTo ( CURLEVEL() -> Background_Song_Name );
+
+    game_status = old_status;
+
     if ( LeaderColor == YourColor )
 	return TRUE;
     else
 	return FALSE;
-    
+
 }; // int Takeover( int enemynum ) 
 
 
-/* ----------------------------------------------------------------------
+/**
  * This function does the countdown where you still can changes your
  * color.
- * ---------------------------------------------------------------------- */
+ */
 void
 ChooseColor (void)
 {
     int countdown = 100;  // duration in 1/10 seconds given for color choosing 
     int ColorChosen = FALSE;
     char count_text[80];
+    SDL_Event event;
     
     Uint32 prev_count_tick, count_tick_len;
     
@@ -409,28 +564,58 @@ ChooseColor (void)
 
     prev_count_tick = SDL_GetTicks ();
 
-    while (!ColorChosen)
-    {
+    while (!ColorChosen) {
+
 	// wait for next countdown tick 
-	while ( SDL_GetTicks() < prev_count_tick + count_tick_len ); 
+	while ( SDL_GetTicks() < prev_count_tick + count_tick_len );
 	
 	prev_count_tick += count_tick_len; // set for next tick 
 	
-	if ( RightPressed () || MouseWheelDownPressed() )
-	{
-	    YourColor = VIOLETT;
-	    OpponentColor = GELB;
-	}
-	if (LeftPressed () || MouseWheelUpPressed() )
-	{
-	    YourColor = GELB;
-	    OpponentColor = VIOLETT;
-	}
-	
-	if ( SpacePressed() || MouseLeftPressed() )
-	{
-	    ColorChosen = TRUE;
-	    while ( SpacePressed() || MouseLeftPressed() ) ;
+	while (SDL_PollEvent(&event)) {
+
+	    if (event.type == SDL_QUIT) {
+		Terminate(0);
+	    }
+	    
+	    if (event.type == SDL_MOUSEBUTTONDOWN) {
+		switch(event.button.button) {
+		    //(clever?) hack : mouse wheel up and down behave
+		    //exactly like LEFT and RIGHT arrow, so we mangle the event
+		    case SDL_BUTTON_WHEELUP:
+			event.type = SDL_KEYDOWN;
+			event.key.keysym.sym = SDLK_LEFT;
+			break;
+		    case SDL_BUTTON_WHEELDOWN:
+			event.type = SDL_KEYDOWN;
+			event.key.keysym.sym = SDLK_RIGHT;
+			break;
+		    case SDL_BUTTON_LEFT:
+			ColorChosen = TRUE;
+			break;
+
+		    default: break;
+		}
+	    } 
+	    
+	    /* no else there! (mouse wheel) */
+	    if (event.type == SDL_KEYDOWN) { 
+		switch(event.key.keysym.sym) {
+		    case SDLK_RIGHT:
+			YourColor = VIOLETT;
+			OpponentColor = GELB;
+			break;
+		    case SDLK_LEFT:
+			YourColor = GELB;
+			OpponentColor = VIOLETT;
+			break;
+		    case SDLK_SPACE:
+			ColorChosen = TRUE;
+			break;
+		    default:
+			break;
+		}
+	    }
+	    
 	}
 	
 	countdown--; // Count down 
@@ -438,15 +623,15 @@ ChooseColor (void)
 	
 	ShowPlayground ();
 	to_show_banner (count_text, NULL);
-	our_SDL_flip_wrapper (Screen);
+	our_SDL_flip_wrapper();
 	
 	if (countdown == 0)
 	  ColorChosen = TRUE;
       
     } // while(!ColorChosen) 
-  
-  return;
-  
+
+    while (MouseLeftPressed())
+	SDL_Delay(1);
 }; // void ChooseColor ( void ) 
 
 
@@ -471,6 +656,8 @@ PlayGame (void)
   int up, down, set; 
   int up_counter, down_counter; 
 
+  SDL_Event event;
+
   sprintf (count_text, _("Subliminal"));   /* Make sure a value gets assigned to count_text */
   count_tick_len = 100;   /* countdown in 1/10 second steps */
   move_tick_len  = 60;    /* allow motion at this tick-speed in ms */
@@ -485,25 +672,42 @@ PlayGame (void)
   while (!FinishTakeover)
     {
       cur_time = SDL_GetTicks ();
-      
+
       /* 
        * here we register if there have been key-press events in the
        * "waiting period" between move-ticks :
        */
-      up   = ( up   | UpPressed() ) + MouseWheelUpPressed() ; 
-      down = ( down | DownPressed() ) + MouseWheelDownPressed() ;
+      up   = ( up   | UpPressed() ); 
+      down = ( down | DownPressed() );
       set  = set  | SpacePressed() | MouseLeftPressed();
 
+      while (SDL_PollEvent(&event)) {
+	  if (event.type == SDL_MOUSEBUTTONDOWN) {
+	      switch (event.button.button) {
+		  case SDL_BUTTON_WHEELUP:
+		      up ++;
+		      break;
+		  case SDL_BUTTON_WHEELDOWN:
+		      down ++;
+		      break;
+		  default:
+		      break;
+	      }
+	  } else if (event.type == SDL_KEYDOWN) {
+	      /* allow for a WIN-key that give immedate victory */
+	      event.key.keysym.mod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE); /* We want to ignore "global" modifiers. */
+	      if (event.key.keysym.sym == SDLK_w && (event.key.keysym.mod == (KMOD_LCTRL | KMOD_LALT))) {
+		  LeaderColor = YourColor;   /* simple as that */
+		  return;  /* leave now, to avoid changing of LeaderColor! */
+	      }
+	  } else if (event.type == SDL_QUIT) {
+	      Terminate(0);
+	  }
+      }
+
       if (!up) up_counter = 0;    /* reset counters for released keys */
       if (!down) down_counter =0;
 
-      /* allow for a WIN-key that give immedate victory */
-      if ( WPressed () && CtrlPressed () && AltPressed () )
-	{
-	  LeaderColor = YourColor;   /* simple as that */
-	  return;  /* leave now, to avoid changing of LeaderColor! */
-	} 
-	
       if ( cur_time > prev_count_tick + count_tick_len ) /* time to count 1 down */
 	{
 	  prev_count_tick += count_tick_len;  /* set for next countdown tick */
@@ -577,6 +781,11 @@ PlayGame (void)
 		  CapsuleCountdown[YourColor][0][row] = CAPSULE_COUNTDOWN * 2;
 
 		  Takeover_Set_Capsule_Sound ();
+
+		  if (!NumCapsules[YOU]) {
+		      /* You placed your last capsule ? let's speed up the end */
+		      count_tick_len *= 0.75;
+		  }
 		}	/* if (row > 0 && ... ) */
 	    } /* if ( set ) */
 
@@ -593,7 +802,8 @@ PlayGame (void)
 
       ShowPlayground ();
       to_show_banner (count_text, NULL);
-      our_SDL_flip_wrapper (Screen);
+      our_SDL_flip_wrapper();
+      SDL_Delay(10);
     }	/* while !FinishTakeover */
 
   /* Final contdown */
@@ -613,7 +823,7 @@ PlayGame (void)
       ProcessPlayground ();	/* this has to be done several times to be sure */
       ProcessDisplayColumn ();
       ShowPlayground ();
-      our_SDL_flip_wrapper (Screen);
+      our_SDL_flip_wrapper();
     }	/* while (countdown) */
 
     return;
@@ -799,7 +1009,7 @@ AdvancedEnemyTakeoverMovements (void)
   return;
 }; // AdvancedEnemyTakeoverMovements 
 
-/* ----------------------------------------------------------------------
+/**
  * This function reads in the takeover game elements for later blitting. 
  * It frees previous SDL-surfaces if they were allocated.  T
  * This allows to use this fct also for theme-switching.
@@ -829,7 +1039,7 @@ GetTakeoverGraphics (void)
   TempLoadSurface = our_IMG_load_wrapper ( fp ) ;
   to_blocks = our_SDL_display_format_wrapperAlpha ( TempLoadSurface ); // the surface is converted
   SDL_FreeSurface ( TempLoadSurface );
-  if ( use_open_gl ) flip_image_horizontally ( to_blocks );
+  if ( use_open_gl ) flip_image_vertically ( to_blocks );
 
   // Get the fill-blocks 
   for ( i = 0 ; i < NUM_FILL_BLOCKS ; i++ , curx += FILL_BLOCK_LEN + 2 )
@@ -931,18 +1141,17 @@ ShowPlayground ( void )
     blit_special_background ( TAKEOVER_BACKGROUND_CODE );
     
     blit_tux ( xoffs + DruidStart [ YourColor ] . x ,
-	       yoffs + DruidStart [ YourColor ] . y + 30 , 0 );
+	       yoffs + DruidStart [ YourColor ] . y + 30);
 
-    if ( AllEnemys [ DroidNum ] . Status != INFOUT )
-	PutEnemy (DroidNum, xoffs + DruidStart[!YourColor].x,
+    PutEnemy (cDroid, xoffs + DruidStart[!YourColor].x,
 		  yoffs + DruidStart[!YourColor].y , FALSE , FALSE );
     
     Set_Rect ( Target_Rect, xoffs + LEFT_OFFS_X, yoffs + LEFT_OFFS_Y,
 	       User_Rect.w, User_Rect.h);
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &ToGroundBlocks [ GELB_OBEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ GELB_OBEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ GELB_OBEN ] . surface , NULL , Screen, &Target_Rect );
     
@@ -951,16 +1160,16 @@ ShowPlayground ( void )
     for ( i = 0 ; i < 12 ; i++ )
     {
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( &ToGroundBlocks [ GELB_MITTE ] , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ GELB_MITTE ] , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( ToGroundBlocks [ GELB_MITTE ] . surface , NULL , Screen, &Target_Rect );
 	Target_Rect.y += GROUNDBLOCKHEIGHT;
     } 
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &ToGroundBlocks [ GELB_UNTEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ GELB_UNTEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ GELB_UNTEN ] . surface , NULL , Screen, &Target_Rect );
     
@@ -968,8 +1177,8 @@ ShowPlayground ( void )
     Set_Rect (Target_Rect, xoffs + MID_OFFS_X, yoffs + MID_OFFS_Y,0, 0);
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &ToLeaderBlock , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToLeaderBlock , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToLeaderBlock . surface , NULL , Screen, &Target_Rect);
     
@@ -977,8 +1186,8 @@ ShowPlayground ( void )
     for ( i = 0 ; i < 12 ; i++ , Target_Rect.y += COLUMNBLOCKHEIGHT )
     {
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( &ToColumnBlock , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &ToColumnBlock , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( ToColumnBlock . surface , NULL , Screen, &Target_Rect );
     }
@@ -987,8 +1196,8 @@ ShowPlayground ( void )
     // the right column
     Set_Rect ( Target_Rect , xoffs + RIGHT_OFFS_X , yoffs + RIGHT_OFFS_Y , 0 , 0 );
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &ToGroundBlocks [ VIOLETT_OBEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ VIOLETT_OBEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ VIOLETT_OBEN ] . surface , NULL , Screen , &Target_Rect );
     
@@ -997,30 +1206,30 @@ ShowPlayground ( void )
     for ( i = 0 ; i < 12 ; i++ , Target_Rect.y += GROUNDBLOCKHEIGHT )
     {
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( &ToGroundBlocks [ VIOLETT_MITTE ] , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ VIOLETT_MITTE ] , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( ToGroundBlocks [ VIOLETT_MITTE ] . surface , NULL , Screen, &Target_Rect );
     }
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &ToGroundBlocks [ VIOLETT_UNTEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ VIOLETT_UNTEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ VIOLETT_UNTEN ] . surface , NULL , Screen, &Target_Rect );
     
     // Fill the leader-LED with its color 
     Set_Rect (Target_Rect, xoffs + LEADERLED_X, yoffs + LEADERLED_Y, 0, 0);
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &FillBlocks [ LeaderColor ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &FillBlocks [ LeaderColor ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( FillBlocks [ LeaderColor ] . surface , NULL , Screen, &Target_Rect );
     
     Target_Rect.y += FILL_BLOCK_HEIGHT;
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( &FillBlocks [ LeaderColor ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &FillBlocks [ LeaderColor ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( FillBlocks [ LeaderColor ] . surface , NULL , Screen, &Target_Rect );
     
@@ -1031,8 +1240,8 @@ ShowPlayground ( void )
 		  yoffs + LEDCOLUMN_Y + i*(FILL_BLOCK_HEIGHT+2),
 		0, 0);
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( &FillBlocks [ DisplayColumn [ i ] ] , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &FillBlocks [ DisplayColumn [ i ] ] , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( FillBlocks [ DisplayColumn [ i ] ] . surface , NULL , Screen, &Target_Rect );
     }
@@ -1046,8 +1255,8 @@ ShowPlayground ( void )
 		      yoffs + PlaygroundStart[GELB].y + j * TO_BLOCKHEIGHT, 0, 0);
 	    block = ToPlayground[GELB][i][j] + ActivationMap[GELB][i][j]*TO_BLOCKS;
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( &ToGameBlocks [ block ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &ToGameBlocks [ block ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( ToGameBlocks [ block ] . surface , NULL , Screen, &Target_Rect );
 	}
@@ -1063,8 +1272,8 @@ ShowPlayground ( void )
 	    block = ToPlayground[VIOLETT][i][j]+
 		(NUM_PHASES+ActivationMap[VIOLETT][i][j])*TO_BLOCKS;
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( &ToGameBlocks [ block ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &ToGameBlocks [ block ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( ToGameBlocks [ block ] . surface , NULL , Screen, &Target_Rect );
 	}
@@ -1083,8 +1292,8 @@ ShowPlayground ( void )
 	if ( NumCapsules [ player ] )
 	{
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( &CapsuleBlocks [ color ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &CapsuleBlocks [ color ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( CapsuleBlocks [ color ] . surface , NULL , Screen, &Target_Rect );
 	}
@@ -1094,8 +1303,8 @@ ShowPlayground ( void )
 	    Set_Rect ( Target_Rect , xoffs + LeftCapsulesStart [ color ] . x ,
 		       yoffs + LeftCapsulesStart [ color ] . y + i * CAPSULE_HEIGHT , 0 , 0 );
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( &CapsuleBlocks [ color ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &CapsuleBlocks [ color ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( CapsuleBlocks [ color ] . surface , NULL , Screen, &Target_Rect );
 	} // for capsules 
@@ -1571,11 +1780,11 @@ ProcessPlayground (void)
   return;
 };  // void ProcessPlayground ( void )
 
-/* ---------------------------------------------------------------------- 
+/** 
  * This function sets the correct values for the status column in the
  * middle of the takeover game field.
  * Binking leds are realized here as well.
- * ---------------------------------------------------------------------- */
+ */
 void
 ProcessDisplayColumn (void)
 {
@@ -1657,15 +1866,15 @@ ProcessDisplayColumn (void)
   // depend on the details of the final takeover score.  Therefore we set this
   // resistance factor variable here.
   //
-  Me[0].Current_Victim_Resistance_Factor = 0.2 * ( (float) 12 - abs( ViolettCounter- GelbCounter ) );
+  Me.Current_Victim_Resistance_Factor = 0.2 * ( (float) 12 - abs( ViolettCounter- GelbCounter ) );
 
   return;
 }; // void ProcessDisplayColumn 
 
-/* ---------------------------------------------------------------------- 
+/** 
  * This function does the countdown of the capsules and kills them if 
  * they are too old.
- * ---------------------------------------------------------------------- */
+ */
 void
 ProcessCapsules (void)
 {
@@ -1689,10 +1898,10 @@ ProcessCapsules (void)
 
 }; // void ProcessCapsules ( void )
 
-/* ----------------------------------------------------------------------
+/**
  * This function tells, wether a Column-connection is active or not.
  * It returns TRUE or FALSE accordinly.
- * ---------------------------------------------------------------------- */
+ */
 int
 IsActive (int color, int row)
 {
@@ -1729,10 +1938,10 @@ AnimateCurrents (void)
   return;
 }; // void AnimateCurrents (void)
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 to_show_banner (const char* left, const char* right)
 {
