@@ -1,5 +1,6 @@
 /* 
  *
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
  *
@@ -23,12 +24,12 @@
  *
  */
 
-/* ----------------------------------------------------------------------
+/**
  * This file contains all functions dealing with texts on the screen,
  * that have to be blitted somehow, using bitmaps or OpenGL texturers,
  * but at least strongly rely on graphics concepts, not pure internal
  * text-processing.
- * ---------------------------------------------------------------------- */
+ */
 
 #define _text_c
 
@@ -40,25 +41,16 @@
 #include "global.h"
 #include "SDL_rotozoom.h"
 
-int DisplayTextWithScrolling (char *Text, int startx, int starty, const SDL_Rect *clip , SDL_Surface* ScrollBackground , float text_stretch );
-
-int CharsPerLine;		// line length in chars:  obsolete
-
 // curent text insertion position
 int MyCursorX;
 int MyCursorY;
 
-// buffer for text environment
-int StoreCursorX;
-int StoreCursorY;
+int display_char_disabled_local;
 
-unsigned int StoreTextBG;
-unsigned int StoreTextFG;
-
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 show_backgrounded_label_at_pixel_position ( char* LabelText , int pos_x , int pos_y )
 {
@@ -75,27 +67,26 @@ show_backgrounded_label_at_pixel_position ( char* LabelText , int pos_x , int po
     
 }; // void show_backgrounded_label_at_pixel_position ( char* LabelText , float fill_status , int pos_x , int pos_y )
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 show_backgrounded_label_at_map_position ( char* LabelText , float fill_status , float pos_x , float pos_y , int zoom_is_on )
 {
     int pixel_x, pixel_y;
-    float zoom_factor = (zoom_is_on ? 1.0 / LEVEL_EDITOR_ZOOM_OUT_FACT : 1.0);
+    float zoom_factor = (zoom_is_on ? ONE_OVER_LEVEL_EDITOR_ZOOM_OUT_FACT : 1.0);
     
     translate_map_point_to_screen_pixel ( pos_x , pos_y , &pixel_x, &pixel_y, zoom_factor );
     show_backgrounded_label_at_pixel_position ( LabelText , pixel_x , pixel_y );
     
 }; // void show_backgrounded_label_at_map_position ( char* LabelText , float fill_status , float pos_x , float pos_y )
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
-void
-show_backgrounded_text_rectangle ( char* text , int x , int y , int w , int h )
+ */
+void show_backgrounded_text_rectangle (const char* text , int x , int y , int w , int h )
 {
     SDL_Rect TargetRect;
 
@@ -117,14 +108,15 @@ show_backgrounded_text_rectangle ( char* text , int x , int y , int w , int h )
     
 }; // void show_backgrounded_text_rectangle ( char* text , int x , int y , int w , int h )
 
-/* ----------------------------------------------------------------------
+/**
  * In some cases it will be nescessary to inform the user of something in
  * a big important style.  Then a popup window is suitable, with a mouse
  * button to confirm and make it go away again.
- * ---------------------------------------------------------------------- */
+ */
 void
-GiveMouseAlertWindow( char* WindowText )
+GiveMouseAlertWindow( const char* WindowText )
 {
+    SDL_Event e;
 
     Activate_Conservative_Frame_Computation();
   
@@ -133,56 +125,58 @@ GiveMouseAlertWindow( char* WindowText )
 				       ( 480 - 340 ) / 2 , 
 				       440 , 340 );
 
-    our_SDL_flip_wrapper ( Screen );
-    
-    while ( !EnterPressed()  && !SpacePressed() && !MouseLeftPressed() ) ;
-    while (  EnterPressed()  ||  SpacePressed() || MouseLeftPressed()) ;
+    our_SDL_flip_wrapper();
     
+    while (1) {
+	SDL_WaitEvent(&e);
+	switch (e.type) {
+	    case SDL_KEYDOWN:
+		if (e.key.keysym.sym == SDLK_SPACE || e.key.keysym.sym == SDLK_RETURN || e.key.keysym.sym == SDLK_ESCAPE)
+		    return;
+		break;
+	    case SDL_MOUSEBUTTONDOWN:
+		if (e.button.button == 1)
+		    return;
+		break;
+	}
+    }
 }; // void GiveMouseAlertWindow( char* WindowText )
 
-/* ----------------------------------------------------------------------
+/**
  * 
- * ---------------------------------------------------------------------- */
-void 
+ */
+int 
 CutDownStringToMaximalSize ( char* StringToCut , int LengthInPixels )
 {
     int StringIndex=0;
-    int LengthByNow=0;
     int i;
     
-    if ( TextWidth ( StringToCut ) <= LengthInPixels ) return;
-    
-    while ( StringToCut[ StringIndex ] != 0 )
+    if ( TextWidth ( StringToCut ) <= LengthInPixels ) return FALSE;
+
+    StringIndex = LimitTextWidth( StringToCut, LengthInPixels );
+    if ( StringIndex < 1 ) return FALSE;
+ 
+    for ( i = 0 ; i < 3 ; i ++ )
     {
-	LengthByNow += CharWidth ( GetCurrentFont() , StringToCut [ StringIndex ] ) ;
-	if ( LengthByNow >= LengthInPixels )
+	if ( StringToCut [ StringIndex + i ] != 0 )
 	{
-	    for ( i = 0 ; i < 3 ; i ++ )
-	    {
-		if ( StringToCut [ StringIndex + i ] != 0 )
-		{
-		    StringToCut [ StringIndex + i ] = '.';
-		}
-		else
-		    return;
-	    }
-	    StringToCut [ StringIndex + 3 ] = 0 ;
-	    return;
+	    StringToCut [ StringIndex + i ] = '.';
 	}
-	StringIndex ++;
+	else
+	    return TRUE;
     }
-    
+    StringToCut [ StringIndex + 3 ] = 0 ;
+
+    return TRUE; 
 }; // void CutDownStringToMaximalSize ( char* StringToCut , int LengthInPixels )
 
-/* ----------------------------------------------------------------------
+/**
  * This function assigns a text comment to say for an enemy right after
  * is has been hit.  This can be turned off via a switch in GameConfig.
- * ---------------------------------------------------------------------- */
+ */
 void 
-EnemyHitByBulletText( int Enum )
+EnemyHitByBulletText( enemy * ThisRobot )
 {
-    Enemy ThisRobot=&AllEnemys[ Enum ];
-
     if ( !GameConfig.Enemy_Hit_Text ) return;
     
     ThisRobot->TextVisibleTime=0;
@@ -209,109 +203,9 @@ EnemyHitByBulletText( int Enum )
 	ThisRobot->TextToBeDisplayed=_("Aargh, I got hit.  Ugh, I got a bad feeling...");
 }; // void EnemyHitByBullet( int Enum );
 
-/* ----------------------------------------------------------------------
- * This function assigns a text comment to say for an enemy right after
- * it has bumped into the player.  This can be turned off via a switch in GameConfig.
- * ---------------------------------------------------------------------- */
-void 
-EnemyInfluCollisionText ( int Enum )
-{
-    Enemy ThisRobot=&AllEnemys[ Enum ];
-    
-    if ( !GameConfig.Enemy_Bump_Text ) return;
-    
-    ThisRobot->TextVisibleTime=0;
-    
-    if ( ThisRobot->is_friendly )
-    {
-	ThisRobot->TextToBeDisplayed=_("Ah, good, that we have an open collision avoiding standard, isn't it.");
-    }
-    else
-    {
-	switch ( MyRandom ( 1 ) )
-	{
-	    case 0:
-		ThisRobot->TextToBeDisplayed=_("Hey, I'm from MS! Walk outa my way!");
-		break;
-	    case 1:
-		ThisRobot->TextToBeDisplayed=_("Hey, I know the big MS boss! You better go.");
-		break;
-	}
-    }
-    
-}; // void EnemyInfluCollisionText ( int Enum )
-
-/* ----------------------------------------------------------------------
- * This function assigns a text comment to say for an enemy while it is
- * standing and aiming for player.  This can be turned off via a switch in GameConfig.
- * ---------------------------------------------------------------------- */
-void 
-AddStandingAndAimingText ( int Enum )
-{
-    Enemy ThisRobot=&AllEnemys[ Enum ];
-    
-    if ( !GameConfig.Enemy_Aim_Text ) return;
-    
-    ThisRobot->TextVisibleTime=0;
-    
-    if ( ( fabsf ( Me [ 0 ] . speed . x ) < 1 ) && ( fabsf ( Me [ 0 ] . speed . y ) < 1 ) )
-    {
-	ThisRobot->TextToBeDisplayed=_("Yeah, stay like that, haha.");
-    }
-    else
-    {
-	ThisRobot->TextToBeDisplayed=_("Stand still while I aim at you.");
-    }
-    
-}; // void AddStandingAndAimingText( int Enum )
-
-/* ----------------------------------------------------------------------
- * This function assigns a text comment to say for the influ right after
- * it has ran into an explosion.  This can be turned off via a switch in GameConfig.
- * ---------------------------------------------------------------------- */
-void
-AddInfluBurntText( void )
-{
-    int FinalTextNr;
-    
-    if ( !GameConfig.Influencer_Blast_Text ) return;
-    
-    Me[0].TextVisibleTime=0;
-    
-    FinalTextNr=MyRandom ( 6 );
-    switch ( FinalTextNr )
-    {
-	case 0:
-	    Me[0].TextToBeDisplayed=_("Aaarrgh, aah, that burnt me!");
-	    break;
-	case 1:
-	    Me[0].TextToBeDisplayed=_("Hell, that blast was hot!");
-	    break;
-	case 2:
-	    Me[0].TextToBeDisplayed=_("Ghaart, I hate to stain my chassis like that.");
-	    break;
-	case 3:
-	    Me[0].TextToBeDisplayed=_("Oh no!  I think I've burnt a cable!");
-	    break;
-	case 4:
-	    Me[0].TextToBeDisplayed=_("Oh no, my poor transfer connectors smolder!");
-	    break;
-	case 5:
-	    Me[0].TextToBeDisplayed=_("I hope that didn't melt any circuits!");
-	    break;
-	case 6:
-	    Me[0].TextToBeDisplayed=_("So that gives some more black scars on me ol' dented chassis!");
-	    break;
-	default:
-	    printf("\nError in AddInfluBurntText! That shouldn't be happening.");
-	    Terminate(ERR);
-	    break;
-    }
-}; // void AddInfluBurntText( void )
-
-/* ----------------------------------------------------------------------
+/**
  * This function sets the text cursor used in DisplayText.
- * ---------------------------------------------------------------------- */
+ */
 void
 SetTextCursor ( int x , int y )
 {
@@ -326,31 +220,21 @@ SetTextCursor ( int x , int y )
  * This function scrolls a given text down inside the User-window, 
  * defined by the global SDL_Rect User_Rect
  *
- * startx/y give the Start-position, 
- * EndLine is the last line (?)
- *
  * ----------------------------------------------------------------- */
 int
-ScrollText (char *Text, int startx, int starty, int background_code )
+ScrollText (char *Text, int background_code )
 {
     int Number_Of_Line_Feeds = 0;	// number of lines used for the text
-    char *textpt;			// mobile pointer to the text
-    int InsertLine = starty;
+    int StartInsertLine, InsertLine;
     int speed = +1;
     int maxspeed = 8;
     
     Activate_Conservative_Frame_Computation( );
     
-    if ( background_code != ( -1 ) )
-	blit_special_background ( background_code );
-    
+    SDL_Rect ScrollRect = { User_Rect.x + 10, User_Rect.y, User_Rect.w - 20, User_Rect.h };
     SetCurrentFont( Para_BFont );
-    
-    // count the number of lines in the text
-    textpt = Text;
-    while (*textpt++)
-	if (*textpt == '\n')
-	    Number_Of_Line_Feeds++;
+    StartInsertLine = ScrollRect.y + ScrollRect.h;
+    InsertLine = StartInsertLine; 
     
     while ( !MouseLeftPressed () 
 	    || ( MouseCursorIsOnButton ( SCROLL_TEXT_UP_BUTTON , GetMousePos_x()  , 
@@ -358,8 +242,20 @@ ScrollText (char *Text, int startx, int starty, int background_code )
 	    || ( MouseCursorIsOnButton ( SCROLL_TEXT_DOWN_BUTTON , GetMousePos_x()  , 
 					 GetMousePos_y()  ) ) )
     {
-	track_last_frame_input_status();
-	keyboard_update();
+    if ( background_code != ( -1 ) ) blit_special_background ( background_code );
+    Number_Of_Line_Feeds = DisplayText ( Text , ScrollRect.x , InsertLine , &ScrollRect , TEXT_STRETCH );
+    //--------------------
+    // We might add some buttons to be displayed here, so that, if you don't have
+    // a mouse wheel and don't know about cursor keys, you can still click on these
+    // buttons to control the scrolling speed of the text.
+    //
+    ShowGenericButtonFromList ( SCROLL_TEXT_UP_BUTTON );
+    ShowGenericButtonFromList ( SCROLL_TEXT_DOWN_BUTTON );
+    blit_our_own_mouse_cursor();
+    our_SDL_flip_wrapper();
+    	
+	save_mouse_state();
+	input_handle();
 
 	if ( UpPressed () 
 	     || ( MouseLeftClicked() 
@@ -393,44 +289,24 @@ ScrollText (char *Text, int startx, int starty, int background_code )
 		speed = -maxspeed;
 	}
 	
-	SDL_Delay (30);
-	
-	if ( background_code != ( -1 ) )
-	    blit_special_background ( background_code );
-	
-	if ( ! DisplayText ( Text , startx , InsertLine , &User_Rect , TEXT_STRETCH ) )
-	{
-	    // JP: I've disabled this, since with this enabled we won't even
-	    // see a single line of the first section of the briefing.
-	    // But this leads to that we currently NEVER can see the second 
-	    // or third part of the briefing text, cause it will not start
-	    // the new text part when the lower end of the first text part
-	    // is reached.  I don't consider this bug release-critical.
-	    //
-	    // break;  /* Text has been scrolled outside User_Rect */
-	}
-	
 	InsertLine -= speed;
 	
 	//--------------------
-	// We might add some buttons to be displayed here, so that, if you don't have
-	// a mouse wheel and don't know about cursor keys, you can still click on these
-	// buttons to control the scrolling speed of the text.
+	// impose some limit on the amount to scroll away downwards and topwards
 	//
-	ShowGenericButtonFromList ( SCROLL_TEXT_UP_BUTTON );
-	ShowGenericButtonFromList ( SCROLL_TEXT_DOWN_BUTTON );
-	blit_our_own_mouse_cursor();
-	our_SDL_flip_wrapper (Screen);
-	
-	//--------------------
-	// impose some limit on the amount to scroll away downwards
-	//
-	if (InsertLine > GameConfig . screen_height - 10 && (speed < 0))
+	if (InsertLine > StartInsertLine && (speed < 0))
 	{
-	    InsertLine = GameConfig . screen_height - 10;
+	    InsertLine = StartInsertLine;
 	    speed = 0;
 	}
+	if ( InsertLine + (Number_Of_Line_Feeds+1)*(int)(FontHeight(GetCurrentFont())*TEXT_STRETCH) < ScrollRect.y && (speed > 0) )
+	{
+		InsertLine = ScrollRect.y - (Number_Of_Line_Feeds+1)*(int)(FontHeight(GetCurrentFont())*TEXT_STRETCH);
+		speed = 0;
+	}
 	
+	SDL_Delay (30);
+
     } // while !Space_Pressed 
     
     while ( MouseLeftPressed() ); // so that we don't touch again immediately.
@@ -439,129 +315,41 @@ ScrollText (char *Text, int startx, int starty, int background_code )
     
 }; // int ScrollText ( ... )
 
-/*-----------------------------------------------------------------
- * This function is much like DisplayText, but with the main difference,
- * that in case of the whole clipping window filled, the function will
- * display a ---more--- line, wait for a key and then scroll the text
- * further up.
- *
- * @Desc: prints *Text beginning at positions startx/starty, 
- * 
- *	and respecting the text-borders set by clip_rect
- *      -> this includes clipping but also automatic line-breaks
- *      when end-of-line is reached
- * 
- *      if clip_rect==NULL, no clipping is performed
- *      
- *      NOTE: the previous clip-rectange is restored before
- *            the function returns!
- *
- *      NOTE2: this function _does not_ update the screen
- *
- * @Ret: TRUE if some characters where written inside the clip rectangle
- *       FALSE if not (used by ScrollText to know if Text has been scrolled
- *             out of clip-rect completely)
- *-----------------------------------------------------------------*/
-int
-DisplayTextWithScrolling ( char *Text , int startx , int starty , const SDL_Rect *clip , SDL_Surface* Background , float text_stretch )
-{
-    char *tmp;	// mobile pointer to the current position as the text is drawn
-    // SDL_Rect Temp_Clipping_Rect; // adding this to prevent segfault in case of NULL as parameter
-    
-    SDL_Rect store_clip;
-    
-    if ( startx != -1 ) MyCursorX = startx;		
-    if ( starty != -1 ) MyCursorY = starty;
-    
-    SDL_GetClipRect (Screen, &store_clip);  // store previous clip-rect 
-    if (clip)
-	SDL_SetClipRect (Screen, clip);
-    else
-    {
-	clip = & User_Rect;
-    }
-    
-    
-    tmp = Text;			// running text-pointer 
-    
-    while ( *tmp && (MyCursorY < clip->y + clip->h) )
-    {
-	if ( *tmp == '\n' )
-	{
-	    MyCursorX = clip->x;
-	    MyCursorY += FontHeight ( GetCurrentFont() ) * TEXT_STRETCH;
-	}
-	else
-	    DisplayChar (*tmp);
-	tmp++;
-	
-	//--------------------
-	// Here we plant in the question for ---more--- and a key to be pressed,
-	// before we clean the screen and restart displaying text from the top
-	// of the given Clipping rectangle
-	//
-	// if ( ( clip->h + clip->y - MyCursorY ) <= 2 * FontHeight ( GetCurrentFont() ) * TEXT_STRETCH )
-	if ( ( clip->h + clip->y - MyCursorY ) <= 1 * FontHeight ( GetCurrentFont() ) * TEXT_STRETCH )
-	{
-	    DisplayText( "--- more --- more --- \n" , MyCursorX , MyCursorY , clip , 1.0 );
-	    our_SDL_flip_wrapper( Screen );
-	    while ( !SpacePressed() );
-	    while (  SpacePressed() );
-	    our_SDL_blit_surface_wrapper( Background , NULL , Screen , NULL );
-	    MyCursorY = clip->y;
-	    our_SDL_flip_wrapper( Screen );
-	};
-	
-	if ( clip )
-	    ImprovedCheckLineBreak ( tmp , clip , text_stretch );   // dont write over right border 
-	
-    } 
-    
-    SDL_SetClipRect (Screen, &store_clip); // restore previous clip-rect 
-    
-    //--------------------
-    // ScrollText() wants to know if we still wrote something inside the
-    // clip-rectangle, of if the Text has been scrolled out
-    // 
-    if ( clip && ((MyCursorY < clip->y) || (starty > clip->y + clip->h) ))
-	return FALSE;  // no text was written inside clip 
-    else
-	return TRUE; 
-    
-};  // DisplayTextWithScrolling (char *Text, int startx, int starty, const SDL_Rect *clip , SDL_Surface* Background )
-
-/* ----------------------------------------------------------------------
+/**
  * This function sets a new text, that will be displayed in huge font 
  * directly over the combat window for a fixed duration of time, where
  * only the time in midst of combat and with no other windows opened
  * is counted.
- * ---------------------------------------------------------------------- */
+ */
 void
-SetNewBigScreenMessage( char* ScreenMessageText )
+SetNewBigScreenMessage(const char* ScreenMessageText )
 {
-    int i;
+    int i = MAX_BIG_SCREEN_MESSAGES - 1;
 
-    //--------------------
-    // Instead of inserting the next big screen message at some fixed cursor position
-    // we will now just move the other big screen messages down and insert the new
-    // big screen message at the very top position of the row.
-    //
-    for ( i = 1 ; i < MAX_BIG_SCREEN_MESSAGES ; i ++ )
-    {
-	strcpy ( Me [ 0 ] . BigScreenMessage [ MAX_BIG_SCREEN_MESSAGES - i ] ,
-		 Me [ 0 ] . BigScreenMessage [ MAX_BIG_SCREEN_MESSAGES - i - 1 ] );
-	Me [ 0 ] . BigScreenMessageDuration [ MAX_BIG_SCREEN_MESSAGES - i ] = 
-	    Me [ 0 ] . BigScreenMessageDuration [ MAX_BIG_SCREEN_MESSAGES - i - 1 ] ;
-    }
-    strcpy ( Me [ 0 ] . BigScreenMessage [ 0 ] , ScreenMessageText );
-    Me [ 0 ] . BigScreenMessageDuration [ 0 ] = 0 ;
+    /* Free the last message that's going to be overwritten */
+    if ( Me . BigScreenMessage [ i ] )
+	{
+	free ( Me . BigScreenMessage [ i ] );
+	Me . BigScreenMessage [ i ] = NULL;
+	}
+
+    while ( i > 0 )
+	{
+	Me . BigScreenMessage [ i ] = Me . BigScreenMessage [ i - 1 ];
+	Me . BigScreenMessageDuration [ i ] = Me . BigScreenMessageDuration [ i - 1];
+	i --;
+	} 
+
+    Me . BigScreenMessage [ 0 ] = MyMalloc(strlen(ScreenMessageText) + 1);
+    strcpy ( Me . BigScreenMessage [ 0 ], ScreenMessageText );
+    Me . BigScreenMessageDuration [ 0 ] = 0 ;
 
 }; // void SetNewBigScreenMessage( char* ScreenMessageText )
 
-/* ----------------------------------------------------------------------
+/**
  * This function displays the currently defined Bigscreenmessage on the
  * screen.  It will be called by AssembleCombatWindow.
- * ---------------------------------------------------------------------- */
+ */
 void
 DisplayBigScreenMessage( void )
 {
@@ -570,15 +358,15 @@ DisplayBigScreenMessage( void )
     
     for ( i = 0 ; i < GameConfig . number_of_big_screen_messages ; i ++ )
     {
-	if ( Me [ 0 ] . BigScreenMessageDuration [ i ] < GameConfig . delay_for_big_screen_messages )
+	if ( Me . BigScreenMessageDuration [ i ] < GameConfig . delay_for_big_screen_messages )
 	{
 	    SDL_SetClipRect ( Screen , NULL );
 	    CenteredPutStringFont ( Screen , Menu_BFont , next_screen_message_position , 
-				    Me [ 0 ] . BigScreenMessage [ i ]  );
+				    Me . BigScreenMessage [ i ]  );
 	    if ( !GameConfig.Inventory_Visible &&
 		 !GameConfig.SkillScreen_Visible &&
 		 !GameConfig.CharacterScreen_Visible )
-		Me [ 0 ] . BigScreenMessageDuration [ i ]  += Frame_Time();
+		Me . BigScreenMessageDuration [ i ]  += Frame_Time();
 
 	    next_screen_message_position += FontHeight ( Menu_BFont ) ;
 	}
@@ -597,18 +385,21 @@ DisplayBigScreenMessage( void )
  *
  *     NOTE2: this function _does not_ update the screen
  *
- * @Ret: number of lines written if some characters where written inside the clip rectangle
- *       FALSE if not (used by ScrollText to know if Text has been scrolled
- *             out of clip-rect completely)
+ * @Ret: number of lines written (from the first text line up to the last
+ *       displayed line)
  *-----------------------------------------------------------------*/
 int
 DisplayText ( const char *Text, int startx, int starty, const SDL_Rect *clip , float text_stretch )
 {
-    unsigned char *tmp;	// mobile pointer to the current position in the string to be printed
+    char *tmp;	// mobile pointer to the current position in the string to be printed
     SDL_Rect Temp_Clipping_Rect; // adding this to prevent segfault in case of NULL as parameter
     SDL_Rect store_clip;
     short int nblines = 1;
-
+    
+    int kerning = 0;
+    if ( GetCurrentFont()==FPS_Display_BFont || GetCurrentFont()==Blue_BFont || GetCurrentFont()==Red_BFont ) kerning = -2;
+	int tab_width = TABWIDTH * (CharWidth ( GetCurrentFont(), TABCHAR) + kerning);
+	
     //--------------------
     // We position the internal text cursor on the right spot for
     // the first character to be printed.
@@ -628,69 +419,59 @@ DisplayText ( const char *Text, int startx, int starty, const SDL_Rect *clip , f
     // we just set the clipping rect to contain the whole screen.
     //
     if ( clip != NULL )
-    {
+	{
 	SDL_SetClipRect ( Screen , clip );
-    }
+	}
     else
-    {
+	{
 	clip = & Temp_Clipping_Rect;
 	Temp_Clipping_Rect.x=0;
 	Temp_Clipping_Rect.y=0;
 	Temp_Clipping_Rect.w=GameConfig . screen_width;
 	Temp_Clipping_Rect.h=GameConfig . screen_height;
-    }
+	}
+
 
-    
     //--------------------
     // Now we can start to print the actual text to the screen.
     //
     // The running text pointer must be initialized.
     //
-    tmp = (unsigned char*) Text;  // this is no longer a 'const' char*, but only a char*
+    tmp = (char*) Text;  // this is no longer a 'const' char*, but only a char*
     while ( *tmp && ( MyCursorY < clip -> y + clip -> h ) )
-    {
-	if ( *tmp == '\n' )
 	{
-	    MyCursorX = clip->x;
-	    MyCursorY += FontHeight ( GetCurrentFont() ) * text_stretch ;
-	    nblines ++;
-	}
-	else 
+   	if( ( (*tmp == ' ') || (*tmp == '\t') )
+   		  && (ImprovedCheckLineBreak( tmp , clip , text_stretch ) == 1) )   // dont write over right border 
+   	    { /*THE CALL ABOVE HAS DONE THE CARRIAGE RETURN FOR US !!!*/
+   	    nblines ++;
+   	    ++tmp;
+   	    continue;
+   	    }
+
+	if ( *tmp == '\n' )
 	    {
-	    if ( MyCursorY > clip -> y - FontHeight ( GetCurrentFont() ) * text_stretch) 
-      	        DisplayChar (*tmp);
-	    else 
-		{
-                MyCursorX += CharWidth ( GetCurrentFont() , *tmp) ;                                
-		switch ( * tmp ) 
-		    {
-		    case 1: SetCurrentFont(Red_BFont); break;
-		    case 2: SetCurrentFont(Blue_BFont); break;
-		    case 3: SetCurrentFont(FPS_Display_BFont); break;
-		    }
-		}
+	    MyCursorX = clip->x;
+	    MyCursorY += (int)(FontHeight ( GetCurrentFont() ) * text_stretch) ;
+	    nblines ++;
+	    }
+	else if ( *tmp == '\t' )
+	    {
+	    MyCursorX = (int)ceilf( (float)MyCursorX / (float)(tab_width) ) * ( tab_width );
+	    }
+	else
+	    {
+	    if ( MyCursorY <= clip -> y - (int)(FontHeight ( GetCurrentFont() ) * text_stretch)) 
+	        display_char_disabled_local = TRUE;
+	    DisplayChar (*tmp);
+	    display_char_disabled_local = FALSE;
 	    }
 
-	
 	tmp++;
-	
-        if(ImprovedCheckLineBreak( tmp , clip , text_stretch ) == 1)   // dont write over right border 
-		{ /*THE CALL ABOVE HAS DONE THE CARRIAGE RETURN FOR US !!!*/
-		nblines ++;
-		}
-	
-    }
+
+	}
     
     SDL_SetClipRect (Screen, &store_clip); // restore previous clip-rect 
     
-    //--------------------
-    // ScrollText() wants to know if we still wrote something inside the
-    // clip-rectangle, of if the Text has been scrolled out
-    //
-    if ( clip && 
-	 ( ( MyCursorY < clip -> y ) || ( starty > clip -> y + clip -> h ) ) )
-	return FALSE;  // no text was written inside clip 
-    else
 	return nblines; 
     
 }; // int DisplayText(...)
@@ -703,98 +484,93 @@ void
 DisplayChar (unsigned char c)
 {
     
-    if ( c == 1 ) 
-    {
+    if ( c == font_switchto_red[0] ) 
+	{
 	SetCurrentFont ( Red_BFont );
 	return;
-    }
-    else if ( c == 2 ) 
-    {
+	}
+    else if ( c == font_switchto_blue[0] ) 
+	{
 	SetCurrentFont ( Blue_BFont );
 	return;
-    }
-    else if ( c == 3 ) 
-    {
+	}
+    else if ( c == font_switchto_neon[0] ) 
+	{
 	SetCurrentFont ( FPS_Display_BFont );
 	return;
-    }
-    
+	}
+    else if ( c == font_switchto_msgstat[0] ) 
+	{
+	SetCurrentFont ( Messagestat_BFont );
+	return;
+	}
+    else if ( c == font_switchto_msgvar[0] ) 
+	{
+	SetCurrentFont ( Messagevar_BFont );
+	return;
+	}
+
     // stupid kerning hack, to get smooth font, but tighter look
     int kerning = 0;
     if (GetCurrentFont()==FPS_Display_BFont || GetCurrentFont()==Blue_BFont || GetCurrentFont()==Red_BFont) kerning = -2;
-    
-    if( c < ' ' || c > GetCurrentFont()->number_of_chars-1){
-        printf("l: %u of %u \n",c,GetCurrentFont()->number_of_chars);
-       c = '.';
-    }
-	if ( ! display_char_disabled ) 
-		PutChar ( Screen, MyCursorX, MyCursorY, c );
 
-    
-    // DebugPrintf( 0 , "%c" , c );
-    
-    // After the char has been displayed, we must move the cursor to its
-    // new position.  That depends of course on the char displayed.
-    //
+    if( c < ' ' || c > GetCurrentFont()->number_of_chars-1){
+	//printf("l: %u of %u \n",c,GetCurrentFont()->number_of_chars);
+	c = '.';
+    }
+    if ( ! display_char_disabled && ! display_char_disabled_local ) 
+	PutCharFont ( Screen, GetCurrentFont(), MyCursorX, MyCursorY, c );
+
     MyCursorX += CharWidth ( GetCurrentFont() , c) +kerning;
-    
+
 }; // void DisplayChar(...)
 
-/* ----------------------------------------------------------------------
+/**
  * This function checks if the next word still fits in this line
  * of text and initiates a carriage return/line feed if not.
  * Very handy and convenient, for that means it is no longer nescessary
  * to enter \n in the text every time its time for a newline. cool.
  *  
- * The function could perhaps still need a little improvement.  But for
- * now its good enough and improvement enough in comparison to the old
- * CheckLineBreak function.
- *
  * rp: added argument clip, which contains the text-window we're writing in
  *     (formerly known as "TextBorder")
  *
  * ah: added return value : 1 if carriage return was done, FALSE otherwise
- * ---------------------------------------------------------------------- */
+ */
 int
-ImprovedCheckLineBreak (unsigned char* Resttext, const SDL_Rect *clip, float text_stretch )
+ImprovedCheckLineBreak (char* Resttext, const SDL_Rect *clip, float text_stretch )
 {
-    int i;
-    int NeededSpace=0;
-#define MAX_WORD_LENGTH 100
+    int NeededSpace = 0;
+   
+    int kerning = 0;
+    if ( GetCurrentFont()==FPS_Display_BFont || GetCurrentFont()==Blue_BFont || GetCurrentFont()==Red_BFont ) kerning = -2;
     
-  // In case of a space, see if the next word will still fit on the line
-  // and do a carriage return/line feed if not
-    if ( *Resttext == ' ' ) {
-	for ( i = 1 ; i < MAX_WORD_LENGTH ; i ++ ) 
-	{
-	    if ( ( Resttext [ i ] != ' ') && ( Resttext [ i ] != '\n') && ( Resttext [ i ] != 0 ) )
+    if ( *Resttext == ' ' )
+    	NeededSpace = CharWidth ( GetCurrentFont(), ' ') + kerning;
+    else if ( *Resttext == '\t' )
+    	NeededSpace = TABWIDTH * (CharWidth ( GetCurrentFont(), TABCHAR) + kerning);
+    
+	Resttext ++;
+	while ( ( *Resttext != ' ') && ( *Resttext != '\t') && ( *Resttext != '\n') && ( *Resttext != 0 ) )
 	    { 
-		NeededSpace += CharWidth ( GetCurrentFont() , Resttext [ i ] );
-		if ( MyCursorX + NeededSpace > clip->x + clip->w - 10 )
-		{
-		    MyCursorX = clip->x;
-		    MyCursorY += FontHeight ( GetCurrentFont() ) * text_stretch ;
-		    return 1;
-		}
+	    NeededSpace += CharWidth ( GetCurrentFont() , *Resttext ) + kerning;
+	    Resttext ++;
 	    }
-	    else 
-		return FALSE;
+
+    if ( (MyCursorX + NeededSpace) > (clip->x + clip->w) )
+	{
+	MyCursorX = clip->x;
+	MyCursorY += (int)(FontHeight ( GetCurrentFont() ) * text_stretch) ;
+	return 1;
 	}
-    }
-return 0;
+    
+    return 0;
 }; // int ImprovedCheckLineBreak()
 
-/* -----------------------------------------------------------------
- * This function reads a string of "MaxLen" from User-input, and 
- * echos it either to stdout or using graphics-text, depending on 
- * the parameter "echo":
- *                      echo=0    no echo
- *                      echo=1    print using printf
- *                      echo=2    print using graphics-text
+/**
+ * This function reads a string of "MaxLen" from User-input, and
+ * echoes it either using graphics-text
  *
- * values of echo > 2 are ignored and treated like echo=0
- *
- * The function does have some extra complicating elelments coming 
+ * The function does have some extra complicating elelments coming
  * from the potential slowness of machines with pure SDL and no
  * OpenGL.  These machines might not re-blit the whole background
  * fast enough for swiftly typed keystrokes, resulting in some
@@ -804,15 +580,11 @@ return 0;
  * a complete re-blit is done in every cycle.
  *
  * NOTE: MaxLen is the maximal _strlen_ of the string (excl. \0 !)
- * 
+ *
  * @Ret: char *: String is allocated _here_!!!
- *       (but since it isn't such a huge amount of memory and since 
- *        it requires manual user input and therefore can't be done
- *        very often anyway, you can as well forget to free it...)
- * 
+ *
  * ----------------------------------------------------------------- */
-char *
-GetString ( int MaxLen, int echo , int background_code , const char* text_for_overhead_promt )
+char * GetString ( int MaxLen, int background_code , const char* text_for_overhead_promt )
 {
     char *input;		// pointer to the string entered by the user
     int key;          // last 'character' entered 
@@ -821,19 +593,13 @@ GetString ( int MaxLen, int echo , int background_code , const char* text_for_ov
     int x0, y0, height;
     SDL_Rect store_rect, tmp_rect;
     SDL_Surface *store = NULL;
-    
-    if (echo == 1)		/* echo to stdout */
-    {
-	printf ("\nGetString(): sorry, echo=1 currently not implemented!\n");
-	return NULL;
-    }
-    
+
     height = FontHeight (GetCurrentFont());
-    
+
     DisplayText ( text_for_overhead_promt , 50 , 50 , NULL , TEXT_STRETCH );
     x0 = MyCursorX;
     y0 = MyCursorY;
-    
+
     if ( use_open_gl )
 	StoreMenuBackground ( 0 );
     else
@@ -842,16 +608,16 @@ GetString ( int MaxLen, int echo , int background_code , const char* text_for_ov
 	Set_Rect ( store_rect , x0 , y0 , GameConfig . screen_width , height );
 	our_SDL_blit_surface_wrapper (Screen, &store_rect, store, NULL);
     }
-    
+
     // allocate memory for the users input
     input     = MyMalloc (MaxLen + 5);
-    
+
     memset (input, '.', MaxLen);
     input[MaxLen] = 0;
-    
+
     finished = FALSE;
     curpos = 0;
-    
+
     while ( !finished  )
     {
 	if ( use_open_gl )
@@ -864,16 +630,15 @@ GetString ( int MaxLen, int echo , int background_code , const char* text_for_ov
 	    Copy_Rect( store_rect, tmp_rect);
 	    our_SDL_blit_surface_wrapper (store, NULL, Screen, &tmp_rect);
 	}
-	
+
 	x0 = MyCursorX;
 	y0 = MyCursorY;
-	
+
 	PutString ( Screen, x0, y0, input);
-	// PutString ( Screen, -1 , -1 , input);
-	our_SDL_flip_wrapper (Screen);
-	
-	key = getchar_raw ();  
-	
+	our_SDL_flip_wrapper();
+
+	key = getchar_raw (NULL);  
+
 	if ( key == SDLK_RETURN ) 
 	{
 #ifdef __WIN32__
@@ -888,26 +653,30 @@ GetString ( int MaxLen, int echo , int background_code , const char* text_for_ov
 		x0 = MyCursorX;
 		y0 = MyCursorY;
 		PutString ( Screen, x0, y0, input);
-		our_SDL_flip_wrapper (Screen);
+		our_SDL_flip_wrapper();
 	    }
 #endif
 	    input[curpos] = 0;
 	    finished = TRUE;
 	}
-	else if ((key < SDLK_DELETE) && isprint (key) && (curpos < MaxLen) )  
-	{
+	else if ((key < SDLK_DELETE) && isprint (key) && (curpos < MaxLen) ) {
 	    DebugPrintf (3, "isprint() true for keycode: %d ('%c')\n", key, (char)key);
 	    /* printable characters are entered in string */
-	    input[curpos] = (char) key;   
+	    input[curpos] = (char) key;
 	    curpos ++;
 	}
-	else if (key == SDLK_BACKSPACE)
-	{
+	else if ((key <= SDLK_KP9) && (key >= SDLK_KP0) && (curpos < MaxLen) ) {
+	    key -= SDLK_KP0;
+	    key += '0';
+
+	    input[curpos] = (char) key;
+	    curpos++;
+	}
+	else if (key == SDLK_BACKSPACE)	{
 	    if ( curpos > 0 ) curpos --;
 	    input[curpos] = '.';
 	}
-	else if ( key == SDLK_ESCAPE )
-	{
+	else if ( key == SDLK_ESCAPE ) {
 	    while(EscapePressed());
 	    return ( NULL );
 	}
@@ -920,7 +689,7 @@ GetString ( int MaxLen, int echo , int background_code , const char* text_for_ov
     
     return (input);
     
-}; // char* GetString( ... ) 
+};
 
 /* -----------------------------------------------------------------
  * This function reads a string of "MaxLen" from User-input.
@@ -976,7 +745,8 @@ GetEditableStringInPopupWindow ( int MaxLen , char* PopupWindowTitle , char* Def
     DisplayText ( PopupWindowTitle, TargetRect . x, TargetRect . y , &TargetRect , TEXT_STRETCH )  ;
     x0 = MyCursorX;
     y0 = MyCursorY;
-    
+   
+    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL); 
     //--------------------
     // Now we can start the enter-string cycle...
     //
@@ -1018,11 +788,11 @@ GetEditableStringInPopupWindow ( int MaxLen , char* PopupWindowTitle , char* Def
 	CursorRect . w = 8 ;
 	HighlightRectangle ( Screen , CursorRect );
 	
-	our_SDL_flip_wrapper (Screen);
+	our_SDL_flip_wrapper();
 	
-	key = getchar_raw ();  
+	key = getchar_raw (NULL);  
 	
-	if (key == SDLK_RETURN) 
+	if (key == SDLK_RETURN || key == SDLK_KP_ENTER) 
 	{
 	    // input[curpos] = 0;
 	    finished = TRUE;
@@ -1074,40 +844,35 @@ GetEditableStringInPopupWindow ( int MaxLen , char* PopupWindowTitle , char* Def
 		curpos --;
 	    }
 	}
+	else if (key == SDLK_DELETE)
+	{
+	    if ( curpos > 0 ) 
+	    {
+		i = curpos ; 
+		while ( input [ i ] != 0 ) 
+		{
+		    input [ i ] = input [ i + 1] ;
+		    i++;
+		}
+	    }
+	    
+	}
+	else if ((key <= SDLK_KP9) && (key >= SDLK_KP0) && (curpos < MaxLen) ) {
+	    key -= SDLK_KP0;
+	    key += '0';
+
+	    input[curpos] = (char) key;
+	    curpos++;
+	}
 	
     } // while ( ! finished ) 
-    
-    DebugPrintf ( 1, "\n%s():  The final string is: '%s'." , __FUNCTION__ , input );
+   
+    SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL); 
     
     return ( input );
     
 }; // char* GetEditableStringInPopupWindow ( int MaxLen , char* PopupWindowTitle )
 
-/* -----------------------------------------------------------------
- * This function is similar to putchar(), but uses the SDL via the 
- * BFont-fct PutChar() instead.
- *
- * Is sets MyCursor[XY], and allows passing (-1,-1) as coords to indicate
- * using the current cursor position.
- * ----------------------------------------------------------------- */
-int
-putchar_SDL (SDL_Surface *Surface, int x, int y, int c)
-{
-    int ret;
-    if (x == -1) x = MyCursorX;
-    if (y == -1) y = MyCursorY;
-    
-    MyCursorX = x + CharWidth (GetCurrentFont(), c);
-    MyCursorY = y;
-    
-    ret = PutChar (Surface, x, y, c);
-    
-    our_SDL_flip_wrapper (Surface);
-    
-    return (ret);
-}; // int putchar_SDL (SDL_Surface *Surface, int x, int y, int c)
-
-
 /* -----------------------------------------------------------------
  * behaves similarly as gl_printf() of svgalib, using the BFont
  * print function PrintString().
